@@ -76,7 +76,6 @@ static int MODIFY_PAGE_NUM = 1;
 /*
  * Forward declarations
  */
-int send_bm_page(SockHandler *sh);
 
 
 /* -- HTML templates ------------------------------------------------------- */
@@ -373,7 +372,7 @@ char *modifypage_add_url =
 /*
  * Return a new string with spaces changed with &nbsp;
  */
-char *make_one_line_str(char *str)
+static char *make_one_line_str(char *str)
 {
    char *new_str;
    int i, j, n;
@@ -402,7 +401,7 @@ char *make_one_line_str(char *str)
  * Use character stuffing for characters given in 'esc_set'.
  * Return value: New stuffed string.
  */
-gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
+static gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
 {
    gint i;
    gchar *p, *sstr = NULL;
@@ -424,7 +423,7 @@ gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
 /*
  * Given an urlencoded string, return it to the original version.
  */
-void Unencode_str(char *e_str)
+static void Unencode_str(char *e_str)
 {
    char *p, *e;
 
@@ -451,10 +450,10 @@ void Unencode_str(char *e_str)
  * Read a line of text up to the newline character, store it into a newly
  * allocated string and return it.
  */
-char *Get_line(FILE *stream)
+static char *Get_line(FILE *stream)
 {
-   int size = 64;
-   int ch, i;
+   guint i, size = 64;
+   int ch;
    char *buf;
 
    buf = g_new(char, size);
@@ -481,14 +480,14 @@ char *Get_line(FILE *stream)
 /*
  * Send a short message to dillo's status bar.
  */
-int Bmsrv_dpi_send_status_msg(SockHandler *sh, char *str)
+static int Bmsrv_dpi_send_status_msg(SockHandler *sh, char *str)
 {
    int st;
    GString *gstr = g_string_new("");
    gchar *sstr = Stuff_chars(str, "'\"");
 
    g_string_sprintf(gstr, "<dpi cmd='send_status_message' msg='%s'>", sstr);
-   st = sock_handler_write(sh, gstr->str, gstr->len, 1);
+   st = sock_handler_write_str(sh, gstr->str, 1);
    g_string_free(gstr, TRUE);
    g_free(sstr);
    return st;
@@ -546,7 +545,7 @@ static BmSec *Bms_get_sec(gint key)
 /*
  * Add a bookmark
  */
-void Bms_add(gint section, char *url, char *title)
+static void Bms_add(gint section, char *url, char *title)
 {
    BmRec *node;
 
@@ -561,7 +560,7 @@ void Bms_add(gint section, char *url, char *title)
 /*
  * Add a section
  */
-void Bms_sec_add(char *title)
+static void Bms_sec_add(char *title)
 {
    BmSec *node;
 
@@ -574,7 +573,7 @@ void Bms_sec_add(char *title)
 /*
  * Delete a bookmark by its key
  */
-void Bms_del(gint key)
+static void Bms_del(gint key)
 {
    GSList *list;
    BmRec *node;
@@ -595,7 +594,7 @@ void Bms_del(gint key)
 /*
  * Delete a section and its bookmarks by section number
  */
-void Bms_sec_del(gint section)
+static void Bms_sec_del(gint section)
 {
    GSList *list;
    BmSec *sec_node;
@@ -623,7 +622,7 @@ void Bms_sec_del(gint section)
 /*
  * Move a bookmark to another section
  */
-void Bms_move(gint key, gint target_section)
+static void Bms_move(gint key, gint target_section)
 {
    GSList *list;
 
@@ -637,7 +636,7 @@ void Bms_move(gint key, gint target_section)
 /*
  * Update a bookmark title by key
  */
-void Bms_update_title(gint key, gchar *n_title)
+static void Bms_update_title(gint key, gchar *n_title)
 {
    GSList *list;
 
@@ -654,7 +653,7 @@ void Bms_update_title(gint key, gchar *n_title)
 /*
  * Update a section title by key
  */
-void Bms_update_sec_title(gint key, gchar *n_title)
+static void Bms_update_sec_title(gint key, gchar *n_title)
 {
    GSList *list;
 
@@ -672,7 +671,7 @@ void Bms_update_sec_title(gint key, gchar *n_title)
 /*
  * Free all the bookmarks data (bookmarks and sections)
  */
-void Bms_free()
+static void Bms_free(void)
 {
    BmRec *bm_node;
    BmSec *sec_node;
@@ -692,7 +691,7 @@ void Bms_free()
 /*
  * Enforce increasing correlative section numbers with no jumps.
  */
-void Bms_normalize()
+static void Bms_normalize(void)
 {
    BmRec *bm_node;
    BmSec *sec_node;
@@ -730,7 +729,7 @@ void Bms_normalize()
 /*
  * If there's no "bm.txt", create one from "bookmarks.html".
  */
-void Bms_check_import()
+static void Bms_check_import(void)
 {
    gchar *OldBmFile;
    char *cmd1 =
@@ -758,7 +757,7 @@ void Bms_check_import()
 /*
  * Load bookmarks data from a file
  */
-int Bms_load()
+static int Bms_load(void)
 {
    FILE *BmTxt;
    char *buf, *p, *url, *title;
@@ -813,7 +812,7 @@ int Bms_load()
  *   - file timestamp is newer than ours  or
  *   - we haven't loaded anything yet :)
  */
-int Bms_cond_load()
+static int Bms_cond_load(void)
 {
    int st = 0;
    struct stat TimeStamp;
@@ -840,7 +839,7 @@ int Bms_cond_load()
  * Update the bookmarks file from memory contents
  * Return code: { 0:OK, 1:Abort }
  */
-int Bms_save()
+static int Bms_save(void)
 {
    FILE *BmTxt;
    BmRec *bm_node;
@@ -869,7 +868,7 @@ int Bms_save()
    for (list = B_secs; list; list = list->next) {
       sec_node = list->data;
       g_string_sprintf(gstr, ":s%d: %s\n", sec_node->section, sec_node->title);
-      fwrite(gstr->str, gstr->len, 1, BmTxt);
+      fwrite(gstr->str, (size_t)gstr->len, 1, BmTxt);
    }
 
    /* save bookmarks  (section url title) */
@@ -880,7 +879,7 @@ int Bms_save()
          if (bm_node->section == sec_node->section) {
             g_string_sprintf(gstr, "s%d %s %s\n",
                              bm_node->section, bm_node->url, bm_node->title);
-            fwrite(gstr->str, gstr->len, 1, BmTxt);
+            fwrite(gstr->str, (size_t)gstr->len, 1, BmTxt);
          }
       }
    }
@@ -900,7 +899,7 @@ int Bms_save()
 /*
  * Add a new bookmark to DB :)
  */
-int Bmsrv_add_bm(SockHandler *sh, char *url, char *title)
+static int Bmsrv_add_bm(SockHandler *sh, char *url, char *title)
 {
    char *msg="Added bookmark!";
    int section = 0;
@@ -952,7 +951,7 @@ static int Bmsrv_send_reload_request(SockHandler *sh, char *url)
    GString *gstr = g_string_new("");
 
    g_string_sprintf(gstr, "<dpi cmd='reload_request' url='%s'>", url);
-   if (sock_handler_write(sh, gstr->str, gstr->len, 1))
+   if (sock_handler_write_str(sh, gstr->str, 1))
       st = 1;
    g_string_free(gstr, TRUE);
 
@@ -1099,7 +1098,7 @@ static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
 
    if (n_sec) {
       g_string_sprintf(gstr, modifypage_update_title, "Update&nbsp;sections:");
-      sock_handler_write(sh, gstr->str, gstr->len, 0);
+      sock_handler_write_str(sh, gstr->str, 0);
       sock_handler_write_str(sh, modifypage_update_item_header, 0);
       /* send items here */
       p = strchr(url1, '?');
@@ -1110,7 +1109,7 @@ static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
             if ((sec_node = Bms_get_sec(key))) {
                g_string_sprintf(gstr, modifypage_update_item2,
                                 sec_node->section, sec_node->title);
-               sock_handler_write(sh, gstr->str, gstr->len, 0);
+               sock_handler_write_str(sh, gstr->str, 0);
             }
          }
       }
@@ -1119,7 +1118,7 @@ static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
 
    if (n_url) {
       g_string_sprintf(gstr, modifypage_update_title, "Update&nbsp;titles:");
-      sock_handler_write(sh, gstr->str, gstr->len, 0);
+      sock_handler_write_str(sh, gstr->str, 0);
       sock_handler_write_str(sh, modifypage_update_item_header, 0);
       /* send items here */
       p = strchr(url1, '?');
@@ -1130,7 +1129,7 @@ static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
             bm_node = Bms_get(key);
             g_string_sprintf(gstr, modifypage_update_item,
                              bm_node->key, bm_node->title, bm_node->url);
-            sock_handler_write(sh, gstr->str, gstr->len, 0);
+            sock_handler_write_str(sh, gstr->str, 0);
          }
       }
       sock_handler_write_str(sh, modifypage_update_item_footer, 0);
@@ -1152,7 +1151,7 @@ static int Bmsrv_send_modify_answer(SockHandler *sh, char *url)
    g_string_sprintf(gstr, "<dpi cmd='start_send_page' url='%s'>", url);
    //g_print("  cmd_buf=%s\n", gstr->str);
    /* Send dpi command */
-   if (sock_handler_write(sh, gstr->str, gstr->len, 1) != 0) {
+   if (sock_handler_write_str(sh, gstr->str, 1) != 0) {
       g_string_free(gstr, TRUE);
       return 1;
    }
@@ -1285,7 +1284,7 @@ static int Bmsrv_modify_update(SockHandler *sh, char *url)
             /* we have a title/key to change */
             key = strtol(p + 1, NULL, 10);
             if ((q = strchr(p + 1, '&')))
-               title = g_strndup(p + 2 + i, q - (p + 2 + i));
+               title = g_strndup(p + 2 + i, (guint)(q - (p + 2 + i)));
             else
                title = g_strdup(p + 2 + i);
 
@@ -1304,7 +1303,7 @@ static int Bmsrv_modify_update(SockHandler *sh, char *url)
             /* we have a title/key to change */
             key = strtol(p + 5, NULL, 10);
             if ((q = strchr(p + 5, '&')))
-               title = g_strndup(p + 6 + i, q - (p + 6 + i));
+               title = g_strndup(p + 6 + i, (guint)(q - (p + 6 + i)));
             else
                title = g_strdup(p + 6 + i);
 
@@ -1535,7 +1534,7 @@ static int Bmsrv_process_modify_request(SockHandler *sh, char *url)
 /*
  * Send the current bookmarks page (in HTML)
  */
-int send_bm_page(SockHandler *sh)
+static int send_bm_page(SockHandler *sh)
 {
    static GString *gstr = NULL;
    gchar *l_title;
@@ -1611,7 +1610,7 @@ int send_bm_page(SockHandler *sh)
  * Note: Buf is a zero terminated string
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-int Bmsrv_parse_buf(SockHandler *sh, char *Buf, int BufSize)
+static int Bmsrv_parse_buf(SockHandler *sh, char *Buf, size_t BufSize)
 {
    char *msg1="<dpi cmd='chat' msg='Hi browser'>";
    char *msg2="<dpi cmd='chat' msg='Is it worth?'>";
@@ -1718,7 +1717,7 @@ int Bmsrv_parse_buf(SockHandler *sh, char *Buf, int BufSize)
  * (was to delete the local namespace socket),
  *  but this is handled by 'dpid' now.
  */
-void cleanup(void)
+static void cleanup(void)
 {
   /* no cleanup required */
 }
@@ -1726,7 +1725,7 @@ void cleanup(void)
 /*
  * Perform any necessary cleanups upon abnormal termination
  */
-void termination_handler(int signum)
+static void termination_handler(int signum)
 {
   exit(signum);
 }
