@@ -47,7 +47,6 @@
 #  define ceprintf(fmt...)
 #endif
 
-extern int errno;
 pid_t origpid, fpid;
 
 /*---------------------------------------------------------------------------*/
@@ -55,7 +54,7 @@ pid_t origpid, fpid;
 /*
  * Make a new name and place it in 'dl_dest'.
  */
-void make_new_name(gchar **dl_dest, const gchar *url)
+static void make_new_name(gchar **dl_dest, const gchar *url)
 {
    GString *gstr = g_string_new(*dl_dest);
    gint idx = gstr->len;
@@ -78,15 +77,39 @@ void make_new_name(gchar **dl_dest, const gchar *url)
 }
 
 /*
- * Escape "'" as "%27"
+ * Filter '\n', '\r', "%0D" and "%0A" from the authority part of an FTP url.
+ * This helps to avoid a SMTP relaying hack. This filtering could be done
+ * only when port == 25, but if the mail server is listening on another
+ * port it wouldn't work.
+ * Note: AFAIS this should be done by wget.
+ * (copied from ftp.c)
  */
-char *escape_quot(char *str)
+static void ftpurl_filter_smtp_hack(GString *gs_url)
 {
    int i;
-   char *p;
-   GString *gstr = g_string_sized_new(64);
+   char c;
 
-eprintf(" unescaped url: %s\n", str);
+   if (gs_url->len > 6) { /* ftp:// */
+      for (i = 6; (c = gs_url->str[i]) && c != '/'; ++i) {
+         if (c == '\n' || c == '\r') {
+            g_string_erase(gs_url, i--, 1);
+         } else if (c == '%' && gs_url->str[i+1] == '0' &&
+                    (tolower(gs_url->str[i+2]) == 'a' ||
+                     tolower(gs_url->str[i+2]) == 'd')) {
+            g_string_erase(gs_url, i--, 3);
+         }
+      }
+   }
+}
+
+/*
+ * Escape "'" as "%27"
+ * (copied from ftp.c)
+ */
+static GString *Url_escape_quote(const char *str)
+{
+   int i;
+   GString *gstr = g_string_sized_new(64);
 
    for (i = 0; str[i]; ++i) {
       if (str[i] == '\'')
@@ -94,20 +117,17 @@ eprintf(" unescaped url: %s\n", str);
       else
          g_string_append_c(gstr, str[i]);
    }
-   p = gstr->str;
-   g_string_free(gstr, FALSE);
 
-eprintf(" escaped url: %s\n", p);
-
-   return p;
+   return gstr;
 }
 
+
 /*---------------------------------------------------------------------------*/
 
 /*
  * SIGCHLD handler
  */
-void sigchld(int sig)
+static void sigchld(int sig)
 {
    ceprintf("downloads server %d: sigchld\n", origpid);
    fflush(stderr);
@@ -118,7 +138,7 @@ void sigchld(int sig)
 /*
  * Establish SIGCHLD handler
  */
-void est_sigchld(void)
+static void est_sigchld(void)
 {
    struct sigaction act;
    sigset_t block;
@@ -134,7 +154,7 @@ void est_sigchld(void)
 /*
  * Read a single line from a socket and store it in a GString.
  */
-ssize_t readline(int socket, GString ** msg)
+static ssize_t readline(int socket, GString ** msg)
 {
    ssize_t rdlen;
    char buf[BUFLEN], c = 0;
@@ -169,8 +189,8 @@ int main(void)
    //char *wget_cmd = "wget -t 1 -nH -P";  --doesn't rename, +other problems
    char *wget_cmd = "wget -O - ";
 
-   char *url = NULL, *esc_url = NULL, *dl_dest = NULL, *cmd = NULL;
-   GString *gs_dl_cmd, *tag;
+   char *url = NULL, *dl_dest = NULL, *cmd = NULL;
+   GString *gs_dl_cmd, *gs_url, *tag;
    fd_set active_set, selected_set;
    struct timeval tout;
    sigset_t blockSC;
@@ -228,7 +248,7 @@ int main(void)
       sigprocmask(SIG_BLOCK, &blockSC, NULL);
       tag = g_string_new(NULL);
       rdlen = readline(new_socket, &tag);
-      if ((cmd = Get_attr_value(tag->str, tag->len, "cmd")) == NULL) {
+      if ((cmd = Get_attr_value(tag->str, (size_t)tag->len, "cmd")) == NULL) {
          eprintf("Failed to parse 'cmd' in %s\n", tag->str);
          exit(1);
       }
@@ -247,19 +267,19 @@ int main(void)
          FILE *in_stream, *out_stream;
          gchar buf[4096];
          struct stat sb;
-         gint n;
+         size_t n;
 
          origpid = cpid = getpid();
          ppid = getppid();
          ceprintf("\n\ndownloads server child %u: Started- Parent=%d\n", cpid,
                   ppid);
-         if ((url = Get_attr_value(tag->str, tag->len, "url")) == NULL) {
+         if ((url = Get_attr_value(tag->str,(size_t)tag->len, "url")) == NULL){
             eprintf("Failed to parse 'url' in %s\n", tag->str);
             exit(1);
          }
          
-         if ((dl_dest =
-              Get_attr_value(tag->str, tag->len, "destination")) == NULL) {
+         dl_dest = Get_attr_value(tag->str, (size_t)tag->len, "destination");
+         if (dl_dest == NULL) {
             eprintf("Failed to parse 'destination' in %s\n", tag->str);
             exit(1);
          }
@@ -282,13 +302,16 @@ int main(void)
          /* make the download command string */
          gs_dl_cmd = g_string_new(NULL);
          /* escape "'" character for the shell */
-         esc_url = escape_quot(url);
-         g_string_sprintf(gs_dl_cmd, "%s '%s'", wget_cmd, esc_url);
+         gs_url = Url_escape_quote(url);
+         /* avoid malicious SMTP relaying with FTP urls */
+         if (g_strncasecmp(gs_url->str, "ftp:/", 5) == 0)
+            ftpurl_filter_smtp_hack(gs_url);
+         g_string_sprintf(gs_dl_cmd, "%s '%s'", wget_cmd, gs_url->str);
          eprintf(" cmd: %s\n", gs_dl_cmd->str);
          eprintf("  to: %s\n", dl_dest);
 
          g_free(dl_dest);
-         g_free(esc_url);
+         g_string_free(gs_url, TRUE);
          g_free(url);
 
          ceprintf("downloads server child %u:parent=%u, Running: %s\n", cpid,
