@@ -15,12 +15,15 @@
  * With new HTML layout.
  */
 
+#include <pthread.h>
+
 #include <ctype.h>           /* for tolower */
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <sys/un.h>
 #include <dirent.h>
 #include <fcntl.h>
@@ -58,23 +61,37 @@ typedef struct {
    GList *flist;       /* List of files and subdirectories (for sorting) */
 } DilloDir;
 
+typedef struct {
+   SockHandler *sh;
+   gint status;
+   gint old_style;
+   pthread_t thrID;
+   gint done;
+} ClientInfo;
+
 /*
  * Forward references
  */
 static const char *File_content_type(const char *filename);
-static gint File_get_file(const gchar *filename,
+static gint File_get_file(ClientInfo *Client,
+                          const gchar *filename,
                           struct stat *sb,
                           const char *orig_url);
-static gint File_get_dir(const gchar *DirName, const char *orig_url);
+static gint File_get_dir(ClientInfo *Client,
+                         const gchar *DirName,
+                         const char *orig_url);
 static gchar *File_html_escape(const gchar *str);
 
 /*
  * Global variables
  */
-static SockHandler *sh;
+static volatile gint DPIBYE = 0;
+static volatile gint ThreadRunning = 0;
 static gint OLD_STYLE = 0;
-
-
+/* A list for the clients we are serving */
+static GList *Clients = NULL;
+/* a mutex for operations on clients */
+static pthread_mutex_t ClMut;
 
 /*
  * Close a file descriptor, but handling EINTR
@@ -308,7 +325,7 @@ static void File_dillodir_free(DilloDir *Ddir)
 /*
  * Output the string for parent directory
  */
-static void File_print_parent_dir(const char *dirname)
+static void File_print_parent_dir(ClientInfo *Client, const char *dirname)
 {
    if (strcmp(dirname, "/") != 0) {        /* Not the root dir */
       char *parent = g_strdup(dirname);
@@ -322,7 +339,7 @@ static void File_print_parent_dir(const char *dirname)
 
       Cparent = (s1 = File_uri_escape(parent, "%#:' ")) ? s1: parent;
       HCparent = (s2 = File_html_escape(Cparent)) ? s2 : Cparent;
-      sock_handler_printf(sh, 0,
+      sock_handler_printf(Client->sh, 0,
          "<a href='file:%s'>Parent directory</a>", HCparent);
       g_free(s2);
       g_free(s1);
@@ -333,20 +350,20 @@ static void File_print_parent_dir(const char *dirname)
 /*
  * Given a timestamp, output an HTML-formatted date string.
  */
-static void File_print_mtime(time_t mtime)
+static void File_print_mtime(ClientInfo *Client, time_t mtime)
 {
    char *ds = ctime(&mtime);
 
    /* Month, day and {hour or year} */
-   if (OLD_STYLE) {
-      sock_handler_printf(sh, 0, " %.3s %.2s", ds + 4, ds + 8);
+   if (Client->old_style) {
+      sock_handler_printf(Client->sh, 0, " %.3s %.2s", ds + 4, ds + 8);
       if (time(NULL) - mtime > 15811200) {
-         sock_handler_printf(sh, 0, "  %.4s", ds + 20);
+         sock_handler_printf(Client->sh, 0, "  %.4s", ds + 20);
       } else {
-         sock_handler_printf(sh, 0, " %.5s", ds + 11);
+         sock_handler_printf(Client->sh, 0, " %.5s", ds + 11);
       }
    } else {
-      sock_handler_printf(sh, 0,
+      sock_handler_printf(Client->sh, 0,
          "<td>%.3s&nbsp;%.2s&nbsp;%.5s", ds + 4, ds + 8,
          // (more than 6 months old) ? year : hour;
          (time(NULL) - mtime > 15811200) ? ds + 20 : ds + 11);
@@ -356,7 +373,8 @@ static void File_print_mtime(time_t mtime)
 /*
  * Return a HTML-line from file info.
  */
-static void File_info2html(FileInfo *finfo, const char *dirname, gint n)
+static void File_info2html(ClientInfo *Client,
+                           FileInfo *finfo, const char *dirname, gint n)
 {
    gint size;
    char *sizeunits;
@@ -402,10 +420,10 @@ static void File_info2html(FileInfo *finfo, const char *dirname, gint n)
    HCref = (s2 = File_html_escape(Cref)) ? s2 : Cref;
    Hname = (s3 = File_html_escape(name)) ? s3 : name;
 
-   if (OLD_STYLE) {
+   if (Client->old_style) {
       char *dots = ".. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..";
       gint ndots = MAXNAMESIZE - strlen(name);
-      sock_handler_printf(sh, 0,
+      sock_handler_printf(Client->sh, 0,
          "%s<a href='%s'>%s</a>"
          " %s"
          " %-11s%4d %-5s",
@@ -414,15 +432,15 @@ static void File_info2html(FileInfo *finfo, const char *dirname, gint n)
          filecont, size, sizeunits);
 
    } else {
-      sock_handler_printf(sh, 0,
+      sock_handler_printf(Client->sh, 0,
          "<tr align=center %s><td>%s<td align=left><a href='%s'>%s</a>"
          "<td>%s<td>%d&nbsp;%s",
          (n & 1) ? "bgcolor=#dcdcdc" : "",
          S_ISDIR (finfo->mode) ? ">" : " ", HCref, Hname,
          filecont, size, sizeunits);
    }
-   File_print_mtime(finfo->mtime);
-   sock_handler_printf(sh, 0, "\n");
+   File_print_mtime(Client, finfo->mtime);
+   sock_handler_printf(Client->sh, 0, "\n");
 
    g_free(s3);
    g_free(s2);
@@ -432,14 +450,15 @@ static void File_info2html(FileInfo *finfo, const char *dirname, gint n)
 /*
  * Read a local directory and translate it to html.
  */
-static void File_transfer_dir(DilloDir *Ddir, const char *orig_url)
+static void File_transfer_dir(ClientInfo *Client,
+                              DilloDir *Ddir, const char *orig_url)
 {
    gint n;
    GList *list;
    char *s1, *s2, *s3, *Hdirname, *Cdirname, *HCdirname;
 
    /* Send DPI header */
-   sock_handler_printf(sh, 1,
+   sock_handler_printf(Client->sh, 1,
       "<dpi cmd='start_send_page' url='%s'>\n", orig_url);
 
    /* Send page title */
@@ -448,7 +467,7 @@ static void File_transfer_dir(DilloDir *Ddir, const char *orig_url)
    HCdirname = (s2 = File_html_escape(Cdirname)) ? s2 : Cdirname;
    Hdirname = (s3 = File_html_escape(Ddir->dirname)) ? s3 : Ddir->dirname;
 
-   sock_handler_printf(sh, 0,
+   sock_handler_printf(Client->sh, 0,
       "Content-Type: text/html\n\n"
       "<HTML>\n<HEAD>\n <BASE href='file:%s'>\n"
       " <TITLE>file:%s</TITLE>\n</HEAD>\n"
@@ -458,22 +477,22 @@ static void File_transfer_dir(DilloDir *Ddir, const char *orig_url)
    g_free(s2);
    g_free(s1);
 
-   if (OLD_STYLE) {
-      sock_handler_printf(sh, 0, "<pre>\n");
+   if (Client->old_style) {
+      sock_handler_printf(Client->sh, 0, "<pre>\n");
    }
 
    /* Output the parent directory */
-   File_print_parent_dir(Ddir->dirname);
+   File_print_parent_dir(Client, Ddir->dirname);
 
    /* HTML style toggle */
-   sock_handler_printf(sh, 0, 
+   sock_handler_printf(Client->sh, 0,
       "&nbsp;&nbsp;<a href='dpi:/file/toggle'>%%</a>\n");
 
    if (Ddir->flist) {
-      if (OLD_STYLE) {
-         sock_handler_printf(sh, 0, "\n\n");
+      if (Client->old_style) {
+         sock_handler_printf(Client->sh, 0, "\n\n");
       } else {
-         sock_handler_printf(sh, 0,
+         sock_handler_printf(Client->sh, 0,
             "<br><br>\n"
             "<table border=0 cellpadding=1 cellspacing=0"
             " bgcolor=#E0E0E0 width=100%%>\n"
@@ -485,23 +504,23 @@ static void File_transfer_dir(DilloDir *Ddir, const char *orig_url)
             "<td><b>Modified&nbsp;at</b>\n");
       }
    } else {
-      sock_handler_printf(sh, 0, "<br><br>Directory is empty...");
+      sock_handler_printf(Client->sh, 0, "<br><br>Directory is empty...");
    }
 
    /* Output entries */
    for (n = 0, list = Ddir->flist; list; list = g_list_next(list)) {
-      File_info2html(list->data, Ddir->dirname, ++n);
+      File_info2html(Client, list->data, Ddir->dirname, ++n);
    }
 
    if (Ddir->flist) {
-      if (OLD_STYLE) {
-         sock_handler_printf(sh, 0, "</pre>\n");
+      if (Client->old_style) {
+         sock_handler_printf(Client->sh, 0, "</pre>\n");
       } else {
-         sock_handler_printf(sh, 0, "</table>\n");
+         sock_handler_printf(Client->sh, 0, "</table>\n");
       }
    }
 
-   sock_handler_printf(sh, 0, "</BODY></HTML>\n");
+   sock_handler_printf(Client->sh, 0, "</BODY></HTML>\n");
 }
 
 /*
@@ -547,7 +566,7 @@ static const char *File_content_type(const char *filename)
          gchar buf[256];
          ssize_t buf_size;
          if ((buf_size = read(fd, buf, 256)) > 12 ) {
-            ct = File_get_content_type_from_data(buf, buf_size);
+            ct = File_get_content_type_from_data(buf, (size_t)buf_size);
          }
          File_close(fd);
       }
@@ -559,7 +578,8 @@ static const char *File_content_type(const char *filename)
 /*
  * Try to stat the file and determine if it's readable.
  */
-static void File_get(const char *filename, const char *orig_url)
+static void File_get(ClientInfo *Client, const char *filename, 
+                     const char *orig_url)
 {
    int res;
    struct stat sb;
@@ -569,18 +589,18 @@ static void File_get(const char *filename, const char *orig_url)
       res = FILE_NOT_FOUND;
    } else if (S_ISDIR(sb.st_mode)) {
       /* set up for reading directory */
-      res = File_get_dir(filename, orig_url);
+      res = File_get_dir(Client, filename, orig_url);
    } else {
       /* set up for reading a file */
-      res = File_get_file(filename, &sb, orig_url);
+      res = File_get_file(Client, filename, &sb, orig_url);
    }
 
    if (res == FILE_NOT_FOUND) {
-      sock_handler_printf(sh, 1,
+      sock_handler_printf(Client->sh, 1,
          "<dpi cmd='send_status_message' msg='%s Not Found: %s'>",
          S_ISDIR(sb.st_mode) ? "Directory" : "File", filename);
    } else if (res == FILE_NO_ACCESS) {
-      sock_handler_printf(sh, 1,
+      sock_handler_printf(Client->sh, 1,
          "<dpi cmd='send_status_message' msg='"
          "Failed to open the %s %s'>",
          S_ISDIR(sb.st_mode) ? "directory" : "file", filename);
@@ -590,7 +610,8 @@ static void File_get(const char *filename, const char *orig_url)
 /*
  *
  */
-static gint File_get_dir(const gchar *DirName, const char *orig_url)
+static gint File_get_dir(ClientInfo *Client,
+                         const gchar *DirName, const char *orig_url)
 {
    GString *g_dirname;
    DilloDir *Ddir;
@@ -604,7 +625,7 @@ static gint File_get_dir(const gchar *DirName, const char *orig_url)
    Ddir = File_dillodir_new(g_dirname->str);
    g_string_free(g_dirname, TRUE);
    if ( Ddir ) {
-      File_transfer_dir(Ddir, orig_url);
+      File_transfer_dir(Client, Ddir, orig_url);
       File_dillodir_free(Ddir);
       return FILE_OK;
    } else
@@ -614,7 +635,8 @@ static gint File_get_dir(const gchar *DirName, const char *orig_url)
 /*
  * Send the MIME content/type and then send the file itself.
  */
-static gint File_get_file(const gchar *filename,
+static gint File_get_file(ClientInfo *Client,
+                          const gchar *filename,
                           struct stat *sb,
                           const char *orig_url)
 {
@@ -638,11 +660,11 @@ static gint File_get_file(const gchar *filename,
 
 
    /* Send DPI command */
-   sock_handler_printf(sh, 1,
+   sock_handler_printf(Client->sh, 1,
       "<dpi cmd='start_send_page' url='%s'>\n", orig_url);
 
    /* Send HTTP stream */
-   sock_handler_printf(sh, 0,
+   sock_handler_printf(Client->sh, 0,
       "Content-Type: %s\n"
       "Content-length: %ld\n\n",
       ct, sb->st_size);
@@ -650,7 +672,7 @@ static gint File_get_file(const gchar *filename,
    /* Send raw file contents */
    do {
       if ((st = read(fd, buf, LBUF)) > 0) {
-         if (sock_handler_write(sh, buf, st, 0) != 0)
+         if (sock_handler_write(Client->sh, buf, (size_t)st, 0) != 0)
             break;
       } else if (st < 0) {
          perror("[read]");
@@ -726,14 +748,15 @@ static char *File_normalize_path(const char *orig)
       return NULL;
 
    str += 5;
-   if (*str == '\0')
-      return NULL;
 
    /* Skip slashes */
    while (str[0] == '/' && str[1] != '\0' && str[1] == '/')
       str++;
 
-   if (*str == '~') {
+   if (*str == '\0') {
+      /* Give CWD if the string is only "file:" */
+      basename = g_get_current_dir();
+   } else if (*str == '~') {
       if (str[1] == '\0' || str[1] == '/') {
          /* Expand 'tilde' to user's home */
          basename = g_strdup(g_get_home_dir());
@@ -750,7 +773,7 @@ static char *File_normalize_path(const char *orig)
          char *tmp2 = strrchr(tmp1, '/');
 
          if (tmp2) {
-            basename = g_strndup(tmp1, tmp2 - tmp1);
+            basename = g_strndup(tmp1, (guint)(tmp2 - tmp1));
          }
          str += 2;
          g_free(tmp1);
@@ -777,27 +800,191 @@ static char *File_normalize_path(const char *orig)
 /*
  * Set the style flag and ask for a reload, so it shows inmediatly.
  */
-static void File_toggle_html_style(void)
+static void File_toggle_html_style(ClientInfo *Client)
 {
    OLD_STYLE = !OLD_STYLE;
-   sock_handler_printf(sh, 0, "<dpi cmd='reload_request'>");
+   sock_handler_printf(Client->sh, 0, "<dpi cmd='reload_request'>");
 }
 
 /*
  * Perform any necessary cleanups upon abnormal termination
  */
-void termination_handler(int signum)
+static void termination_handler(int signum)
 {
   exit(signum);
 }
 
+
+/* Client handling ----------------------------------------------------------*/
+
+/*
+ * Add a new client to the list.
+ */
+static ClientInfo *File_add_client(gint sock_fd)
+{
+   ClientInfo *NewClient;
+
+   NewClient = g_new(ClientInfo, 1);
+   NewClient->sh = sock_handler_new(sock_fd, sock_fd, 8*1024);
+   NewClient->status = 0;
+   NewClient->done = 0;
+   NewClient->old_style = OLD_STYLE;
+  pthread_mutex_lock(&ClMut);
+   Clients = g_list_append(Clients, NewClient);
+  pthread_mutex_unlock(&ClMut);
+   return NewClient;
+}
+
+/*
+ * Get client record by number
+ */
+static void *File_get_client_n(guint n)
+{
+   void *client;
+
+  pthread_mutex_lock(&ClMut);
+   client = g_list_nth_data(Clients, n);
+  pthread_mutex_unlock(&ClMut);
+
+   return client;
+}
+
+/*
+ * Remove a client from the list.
+ */
+static void File_remove_client_n(guint n)
+{
+   ClientInfo *Client;
+
+  pthread_mutex_lock(&ClMut);
+   Client = g_list_nth_data(Clients, n);
+   Clients = g_list_remove(Clients, (void *)Client);
+  pthread_mutex_unlock(&ClMut);
+
+   _MSG("Closing Socket Handler\n");
+   sock_handler_close(Client->sh);
+   sock_handler_free(Client->sh);
+   g_free(Client);
+}
+
+/*
+ * Return the number of clients.
+ */
+static gint File_num_clients(void)
+{
+   guint n;
+
+  pthread_mutex_lock(&ClMut);
+   n = g_list_length(Clients);
+  pthread_mutex_unlock(&ClMut);
+
+   return n;
+}
+
+/*
+ * Serve this client.
+ * (this function runs on its own thread)
+ */
+static void *File_serve_client(void *data)
+{
+   char *dpip_tag, *cmd = NULL, *url = NULL, *path;
+   ClientInfo *Client = data;
+
+   /* Read the dpi command */
+   dpip_tag = sock_handler_read(Client->sh);
+   MSG("dpip_tag = '%s'\n", dpip_tag);
+
+   if (dpip_tag) {
+      cmd = Get_attr_value(dpip_tag, strlen(dpip_tag), "cmd");
+      if (cmd) {
+         if (strcmp(cmd, "DpiBye") == 0) {
+            DPIBYE = 1;
+         } else {
+            url = Get_attr_value(dpip_tag, strlen(dpip_tag), "url");
+            if (!url)
+               MSG("file.dpi:: Failed to parse 'url'\n");
+         }
+      }
+   }
+   g_free(cmd);
+   g_free(dpip_tag);
+
+   if (!DPIBYE && url) {
+      _MSG("url = '%s'\n", url);
+
+      path = File_normalize_path(url);
+      if (path) {
+         _MSG("path = '%s'\n", path);
+         File_get(Client, path, url);
+      } else if (strcmp(url, "dpi:/file/toggle") == 0) {
+         File_toggle_html_style(Client);
+      } else {
+         MSG("ERROR: URL path was %s\n", url);
+      }
+      g_free(path);
+   }
+   g_free(url);
+
+   /* flag the the transfer finished */
+   Client->done = 1;
+
+   return NULL;
+}
+
+/*
+ * Serve the client queue.
+ * (this function runs on its own thread)
+ */
+static void *File_serve_clients(void *client)
+{
+   /* switch to detached state */
+   pthread_detach(pthread_self());
+
+   while (File_num_clients()) {
+      client = File_get_client_n((guint)0);
+      File_serve_client(client);
+      File_remove_client_n((guint)0);
+   }
+   ThreadRunning = 0;
+
+   return NULL;
+}
+
+/* --------------------------------------------------------------------------*/
+
+/*
+ * Check a fd for activity, with a max timeout.
+ * return value: 0 if timeout, 1 if input available, -1 if error.
+ */
+int File_check_fd(int filedes, unsigned int seconds)
+{
+  int st;
+  fd_set set;
+  struct timeval timeout;
+
+  /* Initialize the file descriptor set. */
+  FD_ZERO (&set);
+  FD_SET (filedes, &set);
+
+  /* Initialize the timeout data structure. */
+  timeout.tv_sec = seconds;
+  timeout.tv_usec = 0;
+
+  do {
+     st = select(FD_SETSIZE, &set, NULL, NULL, &timeout);
+  } while (st == -1 && errno == EINTR);
+
+  return st;
+}
+
+
 int main(void)
 {
-   char *dpip_tag, *cmd, *url, *path;
+   ClientInfo *NewClient;
    struct sockaddr_un spun;
-   int temp_sock_descriptor;
-   int address_size;
-   int DpiBye = 0;
+   gint temp_sock_descriptor;
+   gint address_size, c_st, st = 1;
+   guint i;
 
    /* Arrange the cleanup function for abnormal terminations */
    if (signal (SIGINT, termination_handler) == SIG_IGN)
@@ -809,80 +996,56 @@ int main(void)
 
    MSG("(v.1) accepting connections...\n");
 
+   /* initialize mutex */
+   pthread_mutex_init(&ClMut, NULL);
+
    /* some OSes may need this... */
    address_size = sizeof(struct sockaddr_un);
 
    /* start the service loop */
-   while (1) {
-      if (DpiBye) {
-         /* DpiBye command: Service any pending clients and exit */
-         int oldflags = fcntl (STDIN_FILENO, F_GETFL, 0);
-         fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK | oldflags);
-         temp_sock_descriptor =    
-            accept(STDIN_FILENO, (struct sockaddr *)&spun, &address_size);
-         fcntl(STDIN_FILENO, F_SETFL, oldflags);
-         if (temp_sock_descriptor == -1 && errno == EAGAIN)
-            break; /* Done */
-      } else {
-         temp_sock_descriptor =    
-            accept(STDIN_FILENO, (struct sockaddr *)&spun, &address_size);
+   while (!DPIBYE) {
+      /* wait for a connection */
+      do {
+         c_st = File_check_fd(STDIN_FILENO, 1);
+      } while (c_st == 0 && !DPIBYE);
+      if (c_st < 0) {
+         perror("[select]");
+         break;
       }
+      if (DPIBYE)
+         break;
+
+      temp_sock_descriptor =
+         accept(STDIN_FILENO, (struct sockaddr *)&spun, &address_size);
 
       if (temp_sock_descriptor == -1) {
          perror("[accept]");    
-         exit(1);
-      }
-
-      _MSG("temp_sock_descriptor OK\n");
-
-      /* create the SockHandler structure */
-      sh = sock_handler_new(temp_sock_descriptor,temp_sock_descriptor,8*1024);
-
-      while (1) {
-         /* Read the dpi command */
-         dpip_tag = sock_handler_read(sh);
-         MSG("dpip_tag = '%s'\n", dpip_tag);
-      
-         cmd = Get_attr_value(dpip_tag, strlen(dpip_tag), "cmd");
-         if (strcmp(cmd, "DpiBye") == 0) {
-            DpiBye = 1;
-            break;
-         }
-         g_free(cmd);
-
-         url = Get_attr_value(dpip_tag, strlen(dpip_tag), "url");
-         g_free(dpip_tag);
-
-         _MSG("url = '%s'\n", url);
-      
-         if (!url) {
-            MSG("file.dpi:: Failed to parse 'url'\n");
-            break;
-         }
-      
-         path = File_normalize_path(url);
-         _MSG("path = '%s'\n", path);
-
-         //sleep(20);
-
-         if (path) {
-            File_get(path, url);
-         } else if (strcmp(url, "dpi:/file/toggle") == 0) {
-            File_toggle_html_style();
-         } else {
-            MSG("ERROR: URL path was %s\n", url);
-         }
-      
-         g_free(path);
-         g_free(url);
          break;
       }
 
-      _MSG("Closing Socket Handler\n");
-      sock_handler_close(sh);   
-      sock_handler_free(sh);    
+      /* Create and initialize a new client */
+      NewClient = File_add_client(temp_sock_descriptor);
 
-   }/*while*/
+      if (!ThreadRunning) {
+         ThreadRunning = 1;
+         /* Serve the client from a thread (avoids deadlocks) */
+         if (pthread_create(&NewClient->thrID, NULL,
+                            File_serve_clients, NewClient) != 0) {
+            perror("[pthread_create]");
+            ThreadRunning = 0;
+            break;
+         }
+      }
+   }
 
-   return 0;
+   /* todo: handle a running thread better. */
+   for (i = 0; i < 5 && ThreadRunning; ++i) {
+      MSG("sleep i=%u", i);
+      sleep(i);
+   }
+
+   if (DPIBYE)
+      st = 0;
+   return st;
 }
+
