@@ -82,7 +82,7 @@ static SockHandler *sh;
  * the user-selected alternative.
  * Return: (-1: parse error, 0: window closed, 1-5 alt. number)
  */
-static gint dialog_get_answer_number()
+static gint dialog_get_answer_number(void)
 {
    gint response_number = -1;
    gchar *dpip_tag, *response;
@@ -220,7 +220,7 @@ static void yes_ssl_support(void)
 
    if (exit_error == 0){
       /*Send query we want*/
-      SSL_write(ssl_connection, http_query, strlen(http_query));
+      SSL_write(ssl_connection, http_query, (int)strlen(http_query));
 
       /*Analyse response from server*/
 
@@ -230,7 +230,7 @@ static void yes_ssl_support(void)
       /*Send remaining data*/
 
       while ((retval = SSL_read(ssl_connection, buf, 4096)) > 0 ){
-         sock_handler_write(sh, buf, retval, 0);
+         sock_handler_write(sh, buf, (size_t)retval, 0);
       }
    }
 
@@ -304,7 +304,7 @@ static int get_network_connection(gchar * url)
    }
 
    memset(&address,0,sizeof(address));
-   memcpy((char *)&address.sin_addr, hp->h_addr, hp->h_length);
+   memcpy((char *)&address.sin_addr, hp->h_addr, (size_t)hp->h_length);
    address.sin_family=hp->h_addrtype;
    address.sin_port= htons((u_short)portnum);
 
@@ -363,9 +363,9 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
          /*Either self signed and untrusted*/
          /*Extract CN from certificate name information*/
-         strncpy(buf, (strstr(remote_cert->name, "/CN=") + 4), 
-             strstr(strstr(remote_cert->name, "/CN=") + 4, "/") -
-             strstr(remote_cert->name, "/CN=")- 4);
+         strncpy(buf, (strstr(remote_cert->name, "/CN=") + 4),
+             (size_t)(strstr(strstr(remote_cert->name, "/CN=") + 4, "/") -
+                      strstr(remote_cert->name, "/CN=")- 4));
          /*Add terminating NULL*/
          buf[strstr(strstr(remote_cert->name, "/CN=") + 4, "/") -
              strstr(remote_cert->name, "/CN=")- 3] = 0;
