@@ -10,7 +10,7 @@ int w_dpi_socket_dir(char *dirname, char *sockdir);
 
 int tst_dir(char *dir);
 
-char *mk_sockdir();
+char *mk_sockdir(void);
 
 char *init_sockdir(char *dpi_socket_dir);
 
