@@ -427,7 +427,7 @@ int init_srs_socket(char *sockdir)
    sun_path_len = sizeof(srs_sa.sun_path);
    if ( strlen(srs_name) > sun_path_len) {
       ERRMSG("init_srs_socket", "srs_name is too long", 0);
-      fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
+      fprintf(stderr, "\n - it should be <= %lu chars", (gulong)sun_path_len);
       fprintf(stderr, "\n - srs_name = %s\n", srs_name);
       return(retval);
    }
@@ -489,7 +489,7 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
    dpi_attr->sockpath = g_strconcat(sockdir, "/", dpi_nm, NULL);
    if ( strlen(dpi_attr->sockpath) > sp_len) {
       ERRMSG("init_all_dpi_sockets", "socket path is too long", 0);
-      fprintf(stderr, "\n - it should be <= %d chars", sp_len);
+      fprintf(stderr, "\n - it should be <= %lu chars", (gulong)sp_len);
       fprintf(stderr, "\n - socket path = %s\n", dpi_attr->sockpath);
       return(-1);
    }
@@ -616,8 +616,8 @@ void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
       }
       if ( strlen(dpi_attr_list[i].sockpath) > sun_path_len) {
          ERRMSG("stop_active_dpis", "socket path is too long", 0);
-         fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
-         fprintf(stderr, "\n - socket path = %s\n", dpi_attr_list[i].sockpath);
+         fprintf(stderr,"\n - it should be <= %lu chars",(gulong)sun_path_len);
+         fprintf(stderr,"\n - socket path = %s\n", dpi_attr_list[i].sockpath);
       }
       strncpy(dpi_addr.sun_path, dpi_attr_list[i].sockpath, sun_path_len);
       if (connect(dpi_socket, (struct sockaddr *) &dpi_addr, addr_len) == -1) {
@@ -673,7 +673,7 @@ char *get_message(int sock, char *dpi_tag)
 {
    char *msg;
 
-   msg = a_Misc_get_attr_value(dpi_tag, (int) strlen(dpi_tag), "msg");
+   msg = a_Misc_get_attr_value(dpi_tag, strlen(dpi_tag), "msg");
    if (msg == NULL) {
       ERRMSG("get_message", "failed to parse msg\n", 0);
       (void) CKD_WRITE(sock, tag_prserr);
