@@ -49,6 +49,12 @@
 static Klist_t *ValidIOs = NULL; /* Active IOs list. It holds pointers to
                                   * IOData_t structures. */
 
+/*
+ *  Forward declarations
+ */
+void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
+              void *Data1, void *Data2);
+
 /*
  * Debug procedure...
 
@@ -119,23 +125,23 @@ void a_IO_add_buf(IOData_t *io, void *Buf, size_t BufSize)
       return;
    }
 
-   offset = (io->Buf) ? io->Buf - io->BufStart : 0;
+   offset = (io->Buf) ? (gchar*)io->Buf - (gchar*)io->BufStart : 0;
    io->BufStart = g_realloc(io->BufStart, offset + io->BufSize + BufSize);
-   memcpy(io->BufStart + offset + io->BufSize, Buf, BufSize);
-   io->Buf = io->BufStart + offset;
+   memcpy((gchar*)io->BufStart + offset + io->BufSize, Buf, BufSize);
+   io->Buf = (gchar*)io->BufStart + offset;
    io->BufSize += BufSize;
    io->Flags |= IOFlag_FreeIOBuf;
 }
 
 /*
  * Return transfer buffer.
- */
+ *
 void a_IO_get_buf(IOData_t *io, void **Buf, size_t *BufSize)
 {
    *Buf = io->Buf;
    *BufSize = io->BufSize;
 }
-
+ */
 
 typedef struct {
    pthread_t thrID;        /* Thread id */
@@ -186,8 +192,15 @@ static thr_data_t *IO_thrdata_new(gint FD, void *Key)
  */
 static void IO_thrdata_free(thr_data_t *td)
 {
+   /* EBUSY should not happen: IO_thrdata_free is called after the
+    * thread is done */
+   while (pthread_cond_destroy(&td->cond) == EBUSY) {
+      g_warning("IO_thrdata_free: EBUSY\n");
+      if (pthread_cancel(td->thrID) == ESRCH)
+         break;
+   }
+
    pthread_mutex_destroy(&td->mut);
-   pthread_cond_destroy(&td->cond);
    g_free(td);
 }
 
@@ -232,9 +245,6 @@ static void *IO_write_chunk(void *ptr)
    ssize_t St;
    gint st, done;
 
-   /* switch to detached state */
-   pthread_detach(data->thrID);
-
    _MSG("thr::\n");
    _MSG("      [thrID:%lu]\n", (gulong)data->thrID);
 
@@ -298,6 +308,7 @@ static void *IO_write_chunk(void *ptr)
                st = close(data->FD);
             while (st < 0 && errno == EINTR);
          }
+         g_free(data->Buf1Start);
          data->Done = 1;
          pthread_mutex_unlock(&data->mut);
          _MSG("thr:: LastChunk:%d Done:%d  --Bailing out!\n",
@@ -321,6 +332,15 @@ void a_IO_write_chunk(gint FD, void *Key, void *Buf, size_t BufSize)
 {
    thr_data_t *data;
    gint new_thread = 0;
+   static pthread_attr_t thrATTR;
+   static gint thrATTRInitialized = 0;
+
+   /* set the thread attribute to the detached state */
+   if (!thrATTRInitialized) {
+      pthread_attr_init(&thrATTR);
+      pthread_attr_setdetachstate(&thrATTR, PTHREAD_CREATE_DETACHED);
+      thrATTRInitialized = 1;
+   }
 
    /* Search data node for this FD */
    data = IO_thrdata_find(Key);
@@ -338,7 +358,7 @@ void a_IO_write_chunk(gint FD, void *Key, void *Buf, size_t BufSize)
    pthread_mutex_lock(&data->mut);
     if (Buf) {
        data->Buf2 = g_realloc(data->Buf2, data->Buf2Size + BufSize);
-       memcpy(data->Buf2 + data->Buf2Size, Buf, BufSize);
+       memcpy((gchar*)data->Buf2 + data->Buf2Size, Buf, BufSize);
        data->Buf2Size = data->Buf2Size + BufSize;
     } else {
        data->LastChunk = 1;
@@ -348,7 +368,7 @@ void a_IO_write_chunk(gint FD, void *Key, void *Buf, size_t BufSize)
    pthread_mutex_unlock(&data->mut);
 
    if (new_thread)
-      pthread_create(&data->thrID, NULL, IO_write_chunk, data);
+      pthread_create(&data->thrID, &thrATTR, IO_write_chunk, data);
 }
 
 
@@ -442,8 +462,8 @@ static gboolean IO_read(IOData_t *io)
 
       St = read(io->FD, io->Buf, io->BufSize);
       io->Status = St;
-      DEBUG_MSG(3, "  IO_read: %s [errno %d] [St %d]\n",
-                g_strerror(errno), errno, St);
+      DEBUG_MSG(3, "  IO_read: %s [errno %d] [St %ld]\n",
+                g_strerror(errno), errno, (glong)St);
 
       if ( St < 0 ) {
          /* Error */
@@ -490,8 +510,8 @@ static gboolean IO_write(IOData_t *io)
       St = write(io->FD, io->Buf, io->BufSize);
       io->Status = St;
 
-      DEBUG_MSG(3, "  IO_write: %s [errno %d] [St %d]\n",
-                g_strerror(errno), errno, St);
+      DEBUG_MSG(3, "  IO_write: %s [errno %d] [St %ld]\n",
+                g_strerror(errno), errno, (glong)St);
 
       if ( St < 0 ) {
          /* Error */
@@ -507,7 +527,7 @@ static gboolean IO_write(IOData_t *io)
          io->BufSize -= St;
          io->Buf = (gchar *)io->Buf + St;
          DataPending = TRUE;
-         DEBUG_MSG(4, "  IO_write: Changing io->Buf (%d)\n", St);
+         DEBUG_MSG(4, "  IO_write: Changing io->Buf (%ld)\n", (glong)St);
 
       } else {
          /* All data in buffer written */
@@ -712,7 +732,7 @@ void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             _MSG(" Info=%p Info->LocalKey=%p FD=%d Data1=%p\n",
                  Info, Info->LocalKey, *(int*)Info->LocalKey, Data1);
             a_IO_write_chunk(*(int*)Info->LocalKey, Info,
-                             NULL, (Data1) ? 1 : 0);
+                             NULL, (Data1) ? (size_t)1 : (size_t)0);
             g_free(Info->LocalKey);
             break;
          case OpSend:
@@ -720,7 +740,7 @@ void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             /* this part submits the data to the thread */
             DataBuf *dbuf = Data1;
             a_IO_write_chunk(*(int*)Info->LocalKey, Info,
-                             dbuf->Buf, dbuf->Size);
+                             dbuf->Buf, (size_t)dbuf->Size);
             break;
             }
          case OpAbort:
