@@ -3,16 +3,6 @@
 
 #include "../chain.h"
 #include "../url.h"
-#include "IO.h"
-
-
-/*
- * URL opener codes
- */
-#define URL_Http
-#define URL_File
-#define URL_Plugin
-#define URL_None
 
 
 #ifdef __cplusplus
@@ -39,15 +29,13 @@ gchar *a_Http_make_query_str(const DilloUrl *url, gboolean use_proxy);
 
 void a_Http_ccc (int Op, int Branch, int Dir, ChainLink *Info,
                  void *Data1, void *Data2);
-void a_File_ccc (int Op, int Branch, int Dir, ChainLink *Info,
-                 void *Data1, void *Data2);
 void a_About_ccc(int Op, int Branch, int Dir, ChainLink *Info,
                  void *Data1, void *Data2);
 void a_IO_ccc   (int Op, int Branch, int Dir, ChainLink *Info,
                  void *Data1, void *Data2);
 void a_Dpi_ccc  (int Op, int Branch, int Dir, ChainLink *Info,
                  void *Data1, void *Data2);
-void a_Dpi_bye_dpid();
+void a_Dpi_bye_dpid(void);
 
 
 #ifdef __cplusplus
