@@ -11,6 +11,7 @@
  */
 
 #include <config.h>
+#include "IO.h"
 #include "Url.h"
 #include "../nav.h"
 #include "../web.h"
@@ -147,10 +148,6 @@ static char *Splash=
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
-"     <a href='http://www.thejemreport.com/index.php'>TheJemReport</a>\n"
-"    <tr>\n"
-"    <td>&nbsp;&nbsp;\n"
-"    <td>\n"
 "     <a href='http://www.theregister.co.uk/index.html'>The Register</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
@@ -208,14 +205,17 @@ static char *Splash=
 " <td>\n"
 "   <table width='100%' border='0' cellspacing='1' cellpadding='3'>\n"
 "   <tr>\n"
-"    <td colspan='1' bgcolor='#CCCCCC'>Essential Reading\n"
+"    <td colspan='1' bgcolor='#CCCCCC'>Essential Readings\n"
 "\n"
 "   <tr>\n"
 "    <td bgcolor='#FFFFFF'>\n"
 "     <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
-"     <table border='0' cellpadding='2'><tr>\n"
-"     <td>&nbsp;&nbsp;\n"
+"     <table border='0' cellpadding='2'>\n"
+"     <tr><td>&nbsp;&nbsp;\n"
 "     <td><a href='http://www.violence.de'>Peace&amp;Violence</a>\n"
+"     <tr><td>&nbsp;&nbsp;\n"
+"     <td><a href='http://www.fsf.org/philosophy/right-to-read.html'>"
+"      Right to Read</a>\n"
 "     </table>\n"
 "     </table>\n"
 "   </table>\n"
@@ -263,28 +263,21 @@ static char *Splash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  27 October 2004\n"
+"  15 June 2005\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"  The new dillo-0.8.3 comes with a parser that's able to use\n"
-"W3C's semantics plus some heuristics. This improves the rendering\n"
-"of sites that deliver non-valid HTML. It is expected to\n"     
-"provide for better results than the venerable dillo-0.7.3,\n"
-"while keeping the useful diagnostic messages of the "
-"<a href='http://www.dillo.org/help/bug_meter.html'><i>bug meter</i></a>.\n"
+"The  new dillo-" VERSION " is a bugfix release that addresses a couple\n"
+"of security issues. Users are advised to upgrade ASAP.\n"
 "<p>\n"
-"  A new unified API for handling data streams inside dillo plugins\n"
-"was also introduced, and every plugin was ported to it.\n"
-"<p>\n"
-"  Some progress was made with SSL, but as it's still alpha,\n"
-" and very limited, it was disabled by default. Please read the "
-"<a href='http://www.dillo.org/FAQ.html'>FAQ</a> if you want to enable it.\n"
-"<p>\n"
-"  Parsing of cookies was extended, so your favorite site may work now.\n"
+"This release is one of the last on the GTK1-based dillo series.\n"
+"Currently the main development is focused on a "
+"<a href='http://www.fltk.org/'>FLTK2</a>-based dillo prototype.\n"
+"If everything goes as planned, this new branch will become our next release!"
+" (stay tuned).\n"
 "<p>\n"
 "Remember that dillo project uses a release model where every new\n"
 "browser shall be better than the former.\n"
@@ -309,25 +302,11 @@ static char *Splash=
 "  <tr>\n"
 "   <td>\n"
 "<ul>\n"
-"<li>Introduced a light-weight heuristic algorithm over the W3C parsing" 
-" scheme (allows for slightly better rendering: w3c_plus_heuristics=YES).\n"
-"<li>Fixed the width of sized text entry widgets within FORMS.\n"
-"<li>Moved the internal support for \"file:\" URIs into a dpi (filter plugin).\n"
-"<li>Added TABLE-based rendering of directory listings to the new file dpi.\n"
-"<li>Improved the transfer speed for the bookmarks dpi when using"
-" 2.6.x linux.\n"
-"<li>Added a \"dialog\" command to the dpi protocol (dpip). It allows any dpi"
-" to make GUI choice-questions trough Dillo by using simple dpi tags.\n"
-"<li>Designed and implemented a unified API for handling data streams inside"
-" dillo plugins. Servers and filters can use it.\n"
-"<li>Converted the bookmarks, ftp, file, hello and the https prototype dpis"
-" to the new dpiutil API.\n"
-"<li>Replaced the old 'force_visited_color' dillorc option with the new"
-" 'contrast_visited_color' (using a renewed color-choosing algorithm).\n"
-"<li>Fixed cookies to accept four legacy old-date formats for \"Expires\".\n"
-"<li>Added SSL library detection code to configure.in.\n"
-"<li>Made a library-based https dpi prototype,"
-" with certificate authentication.\n"
+"<li>Added a \"small\" dillorc option for panel size.\n"
+"<li>Set \"file:\" to work as URI for current directory.\n"
+"<li>Fixed the shell escaping code in the ftp plugin.\n"
+"<li>Added some checks for sane values in html.c.\n"
+"<li>Added URL filtering to the ftp and downloads dpis to avoid SMTP hacks.\n"
 "</ul>\n"
 "  </table>\n"
 "</table>\n"
@@ -361,7 +340,7 @@ static char *Splash=
 " <li> Dillo behaves very nicely when browsing local files, images, and HTML.\n"
 "   It's also very good for Internet searching (try Google!).\n"
 " <li> This release is mainly intended <strong>for developers</strong>\n"
-"   and <em>advanced users</em>\n"
+"   and <em>advanced users</em>.\n"
 " <li> Frames, Java and Javascript are not supported.\n"
 "</ul>\n"
 "<br>\n"
