@@ -36,8 +36,8 @@
 #include <netdb.h>
 
 #include "../msg.h"
-#include "Url.h"
 #include "IO.h"
+#include "Url.h"
 #include "../misc.h"
 
 //#define DEBUG_LEVEL 2
@@ -89,16 +89,28 @@ static GSList *PendingNodes = NULL;
  */
 
 
+/*
+ * Close a FD handling EINTR
+ */
+static void Dpi_close_fd(gint fd)
+{
+   gint st;
+
+   do
+      st = close(fd);
+   while (st < 0 && errno == EINTR);
+}
+
 /*
  * Task: given a tag and an attribute name, return its value.
  *       (character stuffing is removed here)
  * Return value: the attribute value, or NULL if not present or malformed.
  */
-char *Get_attr_value(char *tag, int tagsize, char *attrname)
+static char *Get_attr_value(char *tag, gint tagsize, char *attrname)
 {
    char *p, *q, *ltag, quote, *start, *val = NULL;
 
-   ltag = g_strndup(tag, tagsize);
+   ltag = g_strndup(tag, (guint)tagsize);
    if ((p = strstr(ltag, attrname)) &&
        (p = strchr(p, '=')) &&
        (p = strpbrk(p, "'\"")) ) {
@@ -107,7 +119,7 @@ char *Get_attr_value(char *tag, int tagsize, char *attrname)
       while ((q = strchr(p, quote)) && q[1] == quote)
          p = q + 2;
       if (q) {
-         val = g_strndup(start, q - start);
+         val = g_strndup(start, (guint)(q - start));
          for (p = q = val; (*q = *p); ++p, ++q)
             if ((*p == '"' || *p == '\'') && p[1] == p[0])
                ++p;
@@ -147,8 +159,8 @@ static void Dpi_conn_data_free(conn_data_t *conn)
 static void Dpi_append_io_buf(conn_data_t *conn, IOData_t *io)
 {
    if (io->Status > 0) {
-      conn->Buf = g_realloc(conn->Buf, io->Status);
-      memcpy(conn->Buf + conn->BufSize, io->Buf, io->Status);
+      conn->Buf = g_realloc(conn->Buf, (gulong)io->Status);
+      memcpy((gchar*)conn->Buf + conn->BufSize, io->Buf, (size_t)io->Status);
       conn->BufSize += io->Status;
       conn->FreeBuf = 0;
    }
@@ -252,7 +264,7 @@ static void Dpi_parse_token(conn_data_t *conn)
       return;
    }
 
-   tag = g_strndup(conn->Tok, conn->TokSize);
+   tag = g_strndup(conn->Tok, (guint)conn->TokSize);
    MSG("Dpi_parse_token: [%s]\n", tag);
 
    cmd = Get_attr_value(conn->Tok, conn->TokSize, "cmd");
@@ -324,7 +336,7 @@ static void Dpi_process_io(int Op, void *Data1, conn_data_t *conn)
  * Start dpid.
  * Return: 0 starting now, 1 Error.
  */
-static gint Dpi_start_dpid()
+static gint Dpi_start_dpid(void)
 {
    pid_t pid;
    gint st_pipe[2], n, ret = 1;
@@ -338,23 +350,30 @@ static gint Dpi_start_dpid()
    if (pid == 0) {
       /* This is the child process.  Execute the command. */
       gchar *path1 = a_Misc_prepend_user_home(".dillo/dpid");
-      close(st_pipe[0]);
+      Dpi_close_fd(st_pipe[0]);
       if (execl(path1, "dpid", NULL) == -1) {
          g_free(path1);
          if (execlp("dpid", "dpid", NULL) == -1) {
             DEBUG_MSG(4, "Dpi_start_dpid (child): %s\n", g_strerror(errno));
-            write(st_pipe[1], "ERROR", 5);
-            close(st_pipe[1]);
+            do
+               n = write(st_pipe[1], "ERROR", 5);
+            while (n == -1 && errno == EINTR);
+            Dpi_close_fd(st_pipe[1]);
             _exit (EXIT_FAILURE);
          }
       }
    } else if (pid < 0) {
       /* The fork failed.  Report failure.  */
       DEBUG_MSG(4, "Dpi_start_dpid: %s\n", g_strerror(errno));
+      /* close the unused pipe */
+      Dpi_close_fd(st_pipe[0]);
+      Dpi_close_fd(st_pipe[1]);
    } else {
       /* This is the parent process, check our child status... */
-      close(st_pipe[1]);
-      n = read(st_pipe[0], buf, 16);
+      Dpi_close_fd(st_pipe[1]);
+      do
+         n = read(st_pipe[0], buf, 16);
+      while (n == -1 && errno == EINTR);
       DEBUG_MSG(2, "Dpi_start_dpid: n = %d\n", n);
       if (n != 5)
          ret = 0;
@@ -369,7 +388,7 @@ static gint Dpi_start_dpid()
  * Make a connection test for a UDS.
  * Return: 0 OK, 1 Not working.
  */
-gint Dpi_check_uds(gchar *uds_name)
+static gint Dpi_check_uds(gchar *uds_name)
 {
    struct sockaddr_un pun;
    gint SockFD, ret = 1;
@@ -384,7 +403,7 @@ gint Dpi_check_uds(gchar *uds_name)
           connect(SockFD, (void*)&pun, D_SUN_LEN(&pun)) == -1) {
          DEBUG_MSG(4, "Dpi_check_uds: %s %s\n", g_strerror(errno), uds_name);
       } else {
-         close(SockFD);
+         Dpi_close_fd(SockFD);
          ret = 0;
       }
    }
@@ -395,7 +414,7 @@ gint Dpi_check_uds(gchar *uds_name)
  * Return the directory where the UDS are in,
  * NULL if it can't be found.
  */
-gchar *Dpi_get_dpid_uds_dir(void)
+static gchar *Dpi_get_dpid_uds_dir(void)
 {
    FILE *in;
    gchar *saved_name_filename;    /*  :)  */
@@ -424,7 +443,7 @@ gchar *Dpi_get_dpid_uds_dir(void)
 /*
  * Return the dpid's UDS name, NULL on failure.
  */
-gchar *Dpi_get_dpid_uds_name(void)
+static gchar *Dpi_get_dpid_uds_name(void)
 {
    gchar *dpid_uds_dir, *dpid_uds_name = NULL;
 
@@ -439,7 +458,7 @@ gchar *Dpi_get_dpid_uds_name(void)
  * Confirm that the dpid is running. If not, start it.
  * Return: 0 running OK, 1 starting (EAGAIN), 2 Error.
  */
-gint Dpi_check_dpid()
+static gint Dpi_check_dpid(void)
 {
    static gint starting = 0;
    gchar *dpid_uds_name;
@@ -484,10 +503,11 @@ gint Dpi_check_dpid()
  *       that connecting each time significantly degrades performance,
  *       an optimized approach can be tried.
  */
-gchar *Dpi_get_server_uds_name(gchar *server_name)
+static gchar *Dpi_get_server_uds_name(gchar *server_name)
 {
    gchar *dpid_uds_dir, *dpid_uds_name = NULL,
          *server_uds_name = NULL;
+   gint st;
 
    g_return_val_if_fail (server_name != NULL, NULL);
    DEBUG_MSG(2, "Dpi_get_server_uds_name:: server_name = [%s]\n", server_name);
@@ -495,8 +515,9 @@ gchar *Dpi_get_server_uds_name(gchar *server_name)
    dpid_uds_dir = Dpi_get_dpid_uds_dir();
    if (dpid_uds_dir) {
       struct sockaddr_un dpid;
-      gint sock, req_sz, rdlen, buflen;
+      gint sock, req_sz, rdlen;
       gchar buf[128], *cmd, *request, *rply;
+      size_t buflen;
 
       /* Get the server's uds name from dpid */
       sock = socket(AF_LOCAL, SOCK_STREAM, 0);
@@ -511,7 +532,10 @@ gchar *Dpi_get_server_uds_name(gchar *server_name)
       request = g_strdup_printf("<dpi cmd='check_server' msg='%s'>",
                                 server_name);
       DEBUG_MSG(2, "[%s]\n", request);
-      if (write(sock, request, strlen(request)) == -1)
+      do
+         st = write(sock, request, strlen(request));
+      while (st < 0 && errno == EINTR);
+      if (st < 0 && errno != EINTR)
          perror("writing request");
       g_free(request);
       shutdown(sock, 1); /* signals no more writes to dpid */
@@ -520,22 +544,27 @@ gchar *Dpi_get_server_uds_name(gchar *server_name)
       rply = NULL;
       buf[0] = '\0';
       buflen = sizeof(buf)/sizeof(buf[0]);
-      for (req_sz = 0;
-            (rdlen = read(sock, buf, buflen)) != 0 && rdlen != -1;
-            req_sz += rdlen) {
-         rply = g_realloc(rply, req_sz + rdlen + 1);
+      for (req_sz = 0; (rdlen = read(sock, buf, buflen)) != 0;
+           req_sz += rdlen) {
+         if (rdlen == -1 && errno == EINTR)
+               continue;
+         if (rdlen == -1) {
+            perror(" ** Dpi_get_server_uds_name **");
+            break;
+         }
+         rply = g_realloc(rply, (guint)(req_sz + rdlen + 1));
          if (req_sz == 0)
             rply[0] = '\0';
-         strncat(rply, buf, rdlen);
+         strncat(rply, buf, (size_t)rdlen);
       }
-      close(sock);
+      Dpi_close_fd(sock);
       DEBUG_MSG(2, "rply = [%s]\n", rply);
 
       /* Parse reply */
-      if (rply) {
-         cmd = Get_attr_value(rply, strlen(rply), "cmd");
+      if (rdlen == 0 && rply) {
+         cmd = Get_attr_value(rply, (gint)strlen(rply), "cmd");
          if (strcmp(cmd, "send_data") == 0)
-            server_uds_name = Get_attr_value(rply, strlen(rply), "msg");
+            server_uds_name = Get_attr_value(rply, (gint)strlen(rply), "msg");
          g_free(cmd);
          g_free(rply);
       }
@@ -785,6 +814,6 @@ void a_Dpi_bye_dpid()
       fprintf(stderr, "%s\n", sa.sun_path);
    }
    (void) write(new_socket, DpiBye_cmd, strlen(DpiBye_cmd));
-   close(new_socket);
+   Dpi_close_fd(new_socket);
 }
 
