@@ -27,8 +27,8 @@
 #include <netinet/in.h>         /* for ntohl and stuff */
 #include <arpa/inet.h>          /* for inet_ntop */
 
-#include "Url.h"
 #include "IO.h"
+#include "Url.h"
 #include "../klist.h"
 #include "../dns.h"
 #include "../cache.h"
@@ -253,13 +253,11 @@ static gboolean
       g_io_channel_close(S->GioCh);
       g_io_channel_unref(S->GioCh);
       S->GioCh = NULL;
-      if (S->addr_list_iter->next) {
-         S->addr_list_iter = S->addr_list_iter->next;
-         if (Http_connect_socket(Info) < 0) {
-            BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
-            Http_socket_free(SKey);
-            a_Chain_fcb(OpAbort, Info, NULL, NULL);
-         }
+      S->addr_list_iter = S->addr_list_iter->next;
+      if (!S->addr_list_iter || Http_connect_socket(Info) < 0) {
+         BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
+         Http_socket_free(SKey);
+         a_Chain_fcb(OpAbort, Info, NULL, NULL);
       }
    } else if ( S->Err ) {
       BW_MSG(S->web, 0, "ERROR: %s", g_strerror(S->Err));
@@ -319,7 +317,7 @@ static int Http_connect_socket(ChainLink *Info)
       socket_len = sizeof(struct sockaddr_in);
       sin->sin_family = dh->af;
       sin->sin_port = S->port ? htons(S->port) : htons(DILLO_URL_HTTP_PORT);
-      memcpy(&sin->sin_addr, dh->data, dh->alen);
+      memcpy(&sin->sin_addr, dh->data, (size_t)dh->alen);
       if (a_Web_valid(S->web) && (S->web->flags & WEB_RootUrl))
          DEBUG_MSG(5, "Connecting to %s\n", inet_ntoa(sin->sin_addr));
       break;
