@@ -312,7 +312,7 @@ static void Cache_entry_remove(CacheData_t *entry, DilloUrl *url)
  */
 static void Cache_stop_clients(DilloUrl *url, gint ByHost)
 {
-   gint i;
+   guint i;
    CacheClient_t *Client;
 
    for (i = 0; (Client = g_slist_nth_data(ClientQueue, i)); ++i) {
@@ -332,7 +332,7 @@ static gint Cache_prepare_reload(DilloUrl *url)
 {
    CacheClient_t *Client;
    DilloWeb *ClientWeb;
-   gint i;
+   guint i;
 
    for (i = 0; (Client = g_slist_nth_data(ClientQueue, i)); ++i){
       if (Cache_client_url_cmp(Client, url) == 0 &&
@@ -476,7 +476,7 @@ gint a_Cache_get_buf(const DilloUrl *Url, gchar **PBuf, gint *BufSize)
 static char *Cache_parse_field(const char *header, const char *fieldname)
 {
    char *field;
-   gint i, j;
+   guint i, j;
 
    for ( i = 0; header[i]; i++ ) {
       /* Search fieldname */
@@ -508,7 +508,7 @@ static char *Cache_parse_field(const char *header, const char *fieldname)
 static GList *Cache_parse_multiple_fields(const char *header,
                                           const char *fieldname)
 {
-   gint i, j;
+   guint i, j;
    GList *fields = NULL;
    char *field;
 
@@ -590,7 +590,7 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
 
    if ( entry->TotalSize > 0 && entry->TotalSize >= entry->ValidSize ) {
       entry->Data = g_malloc(entry->TotalSize);
-      memcpy(entry->Data, (char*)io->Buf+HdrLen, io->Status-HdrLen);
+      memcpy(entry->Data, (char*)io->Buf+HdrLen, (size_t)io->Status-HdrLen);
       /* Prepare next read */
       a_IO_set_buf(io, (char *)entry->Data + entry->ValidSize,
                    entry->TotalSize - entry->ValidSize);
@@ -606,10 +606,10 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
    /* Get Content-Type */
    if ( (Type = Cache_parse_field(header, "Content-Type")) == NULL ) {
      MSG_HTTP("Server didn't send Content-Type in header.\n");
-     Type = g_strdup(a_Misc_get_content_type_from_data(
-                        entry->Data, entry->ValidSize));
+   } else {
+      entry->Type = Type;
+      entry->Flags |= CA_GotContentType;
    }
-   entry->Type = Type;
 }
 
 /*
@@ -799,7 +799,9 @@ void a_Cache_null_client(int Op, CacheClient_t *Client)
  */
 static void Cache_process_queue(CacheData_t *entry)
 {
-   gint i, st;
+   guint i;
+   gint st;
+   const gchar *Type;
    CacheClient_t *Client;
    DilloWeb *ClientWeb;
    BrowserWindow *Client_bw = NULL;
@@ -808,12 +810,19 @@ static void Cache_process_queue(CacheData_t *entry)
 
    if ( Busy )
       DEBUG_MSG(5, "FATAL!:*** >>>> Cache_process_queue Caught busy!!!\n");
-   Busy = TRUE;
-   if ( !(entry->Flags & CA_GotHeader) ) {
-      Busy = FALSE;
+   if (!(entry->Flags & CA_GotHeader))
       return;
+   if (!(entry->Flags & CA_GotContentType)) {
+      st = a_Misc_get_content_type_from_data(
+              entry->Data, entry->ValidSize, &Type);
+      if (st == 0 || entry->Flags & CA_GotData) {
+         entry->Type = g_strdup(Type);
+         entry->Flags |= CA_GotContentType;
+      } else
+         return;  /* i.e., wait for more data */
    }
 
+   Busy = TRUE;
    for ( i = 0; (Client = g_slist_nth_data(ClientQueue, i)); ++i ) {
       if ( Client->Url == entry->Url ) {
          ClientWeb = Client->Web;    /* It was a (void*) */
@@ -945,7 +954,7 @@ static void Cache_remove_client_raw(CacheClient_t *Client, gint Key)
  */
 static void Cache_remove_interface_clients(const DilloUrl *Url)
 {
-   gint i;
+   guint i;
    DilloWeb *Web;
    CacheClient_t *Client;
 
