@@ -16,6 +16,7 @@
  */
 
 #include <string.h>
+#include <unistd.h>   /* for pipe */
 
 #include "msg.h"
 #include "capi.h"
@@ -163,7 +164,7 @@ static void Capi_dpi_conn_resume(void)
    for (i = 0; i < DpiConnSize; ++i)
       if (DpiConn[i]->Flags & PENDING) {
          dpi_conn_t *conn = DpiConn[i];
-         dbuf = a_Chain_dbuf_new(conn->datastr, strlen(conn->datastr), 0);
+         dbuf = a_Chain_dbuf_new(conn->datastr,(gint)strlen(conn->datastr), 0);
          a_Capi_ccc(OpSend, 1, BCK, conn->InfoSend, dbuf, NULL);
          g_free(dbuf);
          conn->Flags &= ~PENDING;
@@ -236,8 +237,11 @@ gint a_Capi_url_uses_dpi(gchar *url_str, gchar **server_ptr)
 
    if (g_strncasecmp(url_str, "dpi:/", 5) == 0) {
       /* dpi prefix, get this server's name */
-      p = strchr(url_str + 5, '/');
-      server = p ? g_strndup(url_str + 5, p - url_str - 5) : g_strdup("?");
+      if ((p = strchr(url_str + 5, '/')) != NULL)
+         server = g_strndup(url_str + 5, (guint)(p - url_str - 5));
+      else
+         server = g_strdup("?");
+
       if (strcmp(server, "bm") == 0) {
          g_free(server);
          server = g_strdup("bookmarks");
@@ -259,7 +263,7 @@ gint a_Capi_url_uses_dpi(gchar *url_str, gchar **server_ptr)
  * Build the dpip command tag, according to URL and server.
  * todo: make it PROXY-aware (AFAIS, it should be easy)
  */
-gchar *a_Capi_dpi_build_cmd(DilloUrl *url, gchar *server)
+static gchar *Capi_dpi_build_cmd(DilloUrl *url, gchar *server)
 {
    gchar *cmd, *http_query, *s_http_query, *s_url;
 
@@ -318,7 +322,7 @@ gint a_Capi_open_url(DilloWeb *web, CA_Callback_t Call, void *CbData)
 
       if (safe && reload) {
          /* Send dpip command */
-         cmd = a_Capi_dpi_build_cmd(web->url, server);
+         cmd = Capi_dpi_build_cmd(web->url, server);
          a_Capi_dpi_send_cmd(web, web->bw, cmd, server, 1);
          g_free(cmd);
 
@@ -377,7 +381,7 @@ gint a_Capi_dpi_send_cmd(DilloWeb *web, void *bw, char *cmd, char *server,
       conn = Capi_dpi_conn_find(server);
       if (conn) {
          /* found */
-         dbuf = a_Chain_dbuf_new(cmd, strlen(cmd), 0);
+         dbuf = a_Chain_dbuf_new(cmd, (gint)strlen(cmd), 0);
          a_Capi_ccc(OpSend, 1, BCK, conn->InfoSend, dbuf, NULL);
          g_free(dbuf);
       } else {
@@ -492,9 +496,9 @@ void a_Capi_ccc(int Op, int Branch, int Dir, ChainLink *Info,
                a_Chain_bcb(OpStop, Info, conn->url, NULL);
 
             } else if (strcmp(Data2, "send_status_message") == 0) {
-               a_Interface_msg(conn->bw, Data1);
+               a_Interface_msg(conn->bw, "%s", Data1);
             } else if (strcmp(Data2, "chat") == 0) {
-               a_Interface_msg(conn->bw, Data1);
+               a_Interface_msg(conn->bw, "%s", Data1);
                a_Bookmarks_chat_add(NULL, NULL, Data1);
             } else if (strcmp(Data2, "dialog") == 0) {
                a_Dpiapi_dialog(conn->bw, conn->server, Data1);
