@@ -1,6 +1,8 @@
 #ifndef __COMMANDS_H__
 #define __COMMANDS_H__
 
+#include "gtk_ext_button.h"
+
 void a_Commands_new_callback(GtkWidget * widget, gpointer client_data);
 void a_Commands_openfile_callback (GtkWidget *widget, gpointer client_data);
 void a_Commands_openurl_callback (GtkWidget *widget, gpointer client_data);
@@ -9,14 +11,20 @@ void a_Commands_close_callback(GtkWidget * widget, gpointer client_data);
 void a_Commands_exit_callback (GtkWidget *widget, gpointer client_data);
 
 void a_Commands_viewsource_callback (GtkWidget *widget, gpointer client_data);
+void a_Commands_view_page_bugs_callback (GtkWidget *button,
+                                         gpointer client_data);
 void a_Commands_selectall_callback (GtkWidget *widget, gpointer client_data);
 void a_Commands_findtext_callback (GtkWidget *widget, gpointer client_data);
 void a_Commands_print_callback (GtkWidget *widget, gpointer client_data);
 
-void a_Commands_navpress_callback (
-        GtkWidget *widget, GdkEventButton *event, gpointer client_data);
-void a_Commands_historypress_callback(
-        GtkWidget *widget, GdkEventButton *event, gpointer client_data);
+GtkMenu *a_Commands_forw_button_menu_creator_callback(GtkExtButton *button,
+                                                      gpointer client_data);
+GtkMenu *a_Commands_back_button_menu_creator_callback(GtkExtButton *button,
+                                                      gpointer client_data);
+void a_Commands_history_callback_same_bw(GtkWidget *widget,
+                                         gpointer client_data);
+void a_Commands_history_callback_new_bw(GtkWidget *widget,
+                                        gpointer client_data);
 
 void a_Commands_back_callback (GtkWidget *widget, gpointer client_data);
 void a_Commands_forw_callback (GtkWidget *widget, gpointer client_data);
@@ -30,13 +38,13 @@ void a_Commands_addbm_callback (GtkWidget *widget, gpointer client_data);
 void a_Commands_viewbm_callback (GtkWidget *widget, gpointer client_data);
 
 void a_Commands_helphome_callback (GtkWidget *widget, gpointer client_data);
-void a_Commands_manual_callback (GtkWidget *widget, gpointer client_data);
 
+void a_Commands_open_link_callback(GtkWidget *widget, gpointer client_data);
 void a_Commands_open_link_nw_callback(GtkWidget *widget, gpointer client_data);
+void a_Commands_select_popup_url_callback(GtkWidget *widget, gpointer data);
 
-void a_Commands_set_selection_callback(GtkWidget *widget,gpointer client_data);
-void a_Commands_give_selection_callback(GtkWidget *widget,
-      GtkSelectionData *data, guint info, guint time);
-gint a_Commands_clear_selection_callback(GtkWidget *, GdkEventSelection *);
+void a_Commands_ob_w3c_callback(GtkWidget *widget, gpointer client_data);
+void a_Commands_ob_wdg_callback(GtkWidget *widget, gpointer client_data);
+void a_Commands_ob_info_callback(GtkWidget *widget, gpointer client_data);
 
 #endif /* __COMMANDS_H__ */
