@@ -21,6 +21,7 @@
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <fcntl.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>       /* for time() and time_t */
@@ -566,7 +567,7 @@ static void Cookies_remove_cookie(CookieData_t *cookie)
 static char *Cookies_parse_attr(char **cookie_str)
 {
    char *str = *cookie_str;
-   int i, end = 0;
+   guint i, end = 0;
    gboolean got_attr = FALSE;
 
    for (i = 0; ; i++) {
@@ -609,11 +610,10 @@ static char *Cookies_parse_value(char **cookie_str,
                                  gboolean broken_syntax,
                                  gboolean keep_quotes)
 {
+   guint i, end;
    char *str = *cookie_str;
-   int i = -1, end = 0;
 
-   while (!end) {
-      ++i;
+   for (i = end = 0; !end; ++i) {
       switch (str[i]) {
       case ' ':
       case '\t':
@@ -658,6 +658,8 @@ static char *Cookies_parse_value(char **cookie_str,
          break;
       }
    }
+   /* keep i as an index to the last char */
+   --i;
 
    if ((str[0] == '\'' || str[0] == '"') && !keep_quotes) {
       return i > 1 ? g_strndup(str + 1, i - 1) : NULL;
@@ -890,7 +892,7 @@ static gboolean Cookies_validate_domain(CookieData_t *cookie,
 static char *Cookies_strip_path(const char *path)
 {
    char *ret;
-   int len;
+   guint len;
 
    if (path) {
       len = strlen(path);
