@@ -44,6 +44,7 @@
 #include "dicache.h"
 #include "dns.h"
 #include "IO/mime.h"
+#include "IO/Url.h"
 #include "prefs.h"
 #include "interface.h"
 #include "dw.h"
@@ -56,8 +57,8 @@
 typedef struct {
    char *sh_opt; /* must be non NULL */
    char *lg_opt; /* must be non NULL */
-   int opt_argc; /* positive: mandatory, negative: optional */
-   int opt_flg;  /* 1 << n */
+   gint opt_argc; /* positive: mandatory, negative: optional */
+   gint opt_flg;  /* 1 << n */
    char *help;   /* can be NULL */
 } CLI_options;
 
@@ -69,7 +70,7 @@ enum {
    DILLO_CLI_LOCAL         = 1 << 5,
    DILLO_CLI_GEOMETRY      = 1 << 6,
    DILLO_CLI_DBG_RENDERING = 1 << 7,
-   DILLO_CLI_ERROR         = 1 << 31
+   DILLO_CLI_ERROR         = 1 << 15
 };
 
 enum {
@@ -85,8 +86,8 @@ gboolean dillo_dbg_rendering = FALSE;
  * Forward declarations
  */
 static void Dillo_check_home_dir(char *dir);
-static guint32 Dillo_get_opt(CLI_options *options, int argc,
-                             char **argv, char ***opt_arg, gint *idx);
+static gint Dillo_get_opt(CLI_options *options, int argc,
+                          char **argv, char ***opt_arg, gint *idx);
 static void Dillo_print_help(CLI_options *options);
 static void Dillo_print_version(void);
 
@@ -102,7 +103,8 @@ static CLI_options Options[] = {
    {"-h", "--help",       0, DILLO_CLI_HELP,
     "  -h, --help             Display this help text and exit."},
    {"-f", "--fullwindow", 0, DILLO_CLI_FULLWINDOW,
-    "  -f, --fullwindow       Start in full window mode."},
+    "  -f, --fullwindow       Start in full window mode: hide address bar,\n"
+    "                         navigation buttons, menu, and status bar."},
    {"-l", "--local",      0, DILLO_CLI_LOCAL,
     "  -l, --local            Don't follow links for this URL(s)."},
    {"-g", "-geometry",    1, DILLO_CLI_GEOMETRY,
@@ -151,6 +153,7 @@ static DilloUrl *Dillo_make_start_url(gchar *str)
       GString *UrlStr = g_string_sized_new(128);
 
       if (url_str[0] == '/') {
+         /* prepend the "file:" protocol specifier. */
          g_string_sprintf(UrlStr, "file:%s", url_str);
       } else {
          cd = g_get_current_dir();
@@ -202,9 +205,9 @@ gint main(int argc, char *argv[])
    DilloUrl *start_url;
    BrowserWindow *bw;
    guint32 xid = 0;
-   guint32 options_got = 0;
+   guint options_got = 0;
    gint idx = 0;
-   guint opt_flg;
+   gint opt_flg;
    gint i;
    char **opt_argv = NULL;
    gint xpos = D_GEOMETRY_DEFAULT_XPOS, ypos = D_GEOMETRY_DEFAULT_YPOS;
@@ -369,13 +372,13 @@ static void Dillo_check_home_dir(char *dir)
  * -1 if an unrecognised option is found or if a mandatory option
  *    argument is not found.
  */
-static guint32
+static gint
 Dillo_get_opt(CLI_options *options, int argc,
               char **argv, char ***opt_argv, gint *cli_idx)
 {
    gint i = 0, opt_flg = 0, n_arg, state;
    static int idx = 1;
-   guint32 ret_val = 0;
+   gint ret_val = 0;
 
    /* Allocate opt_argv */
    if (*opt_argv == NULL)
