@@ -4,6 +4,6 @@
 #include "browser.h"
 #include "web.h"
 
-#define DILLO_HOME "http://dillo.cipsga.org.br/"
+extern gboolean dillo_dbg_rendering;
 
 #endif /* __DILLO_H__ */
