@@ -17,23 +17,11 @@ extern "C" {
 typedef struct _DwBullet      DwBullet;
 typedef struct _DwBulletClass DwBulletClass;
 
-typedef enum
-{
-   DW_BULLET_DISC=1,
-   DW_BULLET_CIRCLE=2,
-   DW_BULLET_SQUARE=3,
-   DW_BULLET_1=4,
-   DW_BULLET_a=5,
-   DW_BULLET_A=6,
-   DW_BULLET_i=7,
-   DW_BULLET_I=8
-} DwBulletType;
-
 struct _DwBullet
 {
    DwWidget widget;
 
-   DwBulletType type;
+   gboolean selected[DW_HIGHLIGHT_NUM_LAYERS];
 };
 
 struct _DwBulletClass
@@ -43,7 +31,7 @@ struct _DwBulletClass
 
 
 GtkType         a_Dw_bullet_get_type    (void);
-DwWidget*       a_Dw_bullet_new         (DwBulletType type);
+DwWidget*       a_Dw_bullet_new         (void);
 
 #ifdef __cplusplus
 }
