@@ -11,7 +11,7 @@
 
 /*
  * This widget imitates the look and behavior of GtkButton.
- * 
+ *
  * NOTE: Unlike in X, we do not get release events, if the pointer has
  * been moved out of the widget with mouse button down (implicit
  * pointer grab, will perhaps be added to Dw). For this reason, the
@@ -22,7 +22,7 @@
  * when the user leaves the button. This is to avoid problems in the
  * situation when the user leaves the button with mouse button down,
  * and releases it outside of the button.
- * 
+ *
  * BUG (reminder): If the user releases the button, it gets somehow a
  * "leave_notify" event. This bug is in the base code, not here,
  * somehow it is assumed that the mouse pointer is out of the viewport
@@ -223,7 +223,7 @@ static void Dw_button_size_request (DwWidget *widget,
    GtkStyleClass *styleclass = widget->viewport->style->klass;
 
    if (button->child) {
-      a_Dw_widget_size_request (button->child, &child_requisition);
+      p_Dw_widget_size_request (button->child, &child_requisition);
       *requisition = child_requisition;
    } else {
       requisition->width = 0;
@@ -250,7 +250,7 @@ static void Dw_button_get_extremes (DwWidget *widget,
    GtkStyleClass *styleclass = widget->viewport->style->klass;
 
    if (button->child) {
-      a_Dw_widget_get_extremes (button->child, &child_extremes);
+      p_Dw_widget_get_extremes (button->child, &child_extremes);
       *extremes = child_extremes;
    } else {
       extremes->min_width = 0;
@@ -285,7 +285,7 @@ static void Dw_button_size_allocate (DwWidget *widget,
          child_allocation.descent -= styleclass->ythickness;
       }
 
-      a_Dw_widget_size_allocate (button->child, &child_allocation);
+      p_Dw_widget_size_allocate (button->child, &child_allocation);
    }
 }
 
@@ -299,7 +299,7 @@ static void Dw_button_set_width (DwWidget *widget,
    DwButton *button = DW_BUTTON (widget);
 
    if (button->child)
-      a_Dw_widget_set_width
+      p_Dw_widget_set_width
          (button->child,
           width - (button->relief ?
                    (2 * widget->viewport->style->klass->xthickness) : 0));
@@ -315,7 +315,7 @@ static void Dw_button_set_ascent (DwWidget *widget,
    DwButton *button = DW_BUTTON (widget);
 
    if (button->child)
-      a_Dw_widget_set_ascent
+      p_Dw_widget_set_ascent
          (button->child,
           ascent  - (button->relief ?
                      widget->viewport->style->klass->ythickness : 0));
@@ -331,7 +331,7 @@ static void Dw_button_set_descent (DwWidget *widget,
    DwButton *button = DW_BUTTON (widget);
 
    if (button->child)
-      a_Dw_widget_set_descent
+      p_Dw_widget_set_descent
          (button->child,
           descent - (button->relief ?
                      widget->viewport->style->klass->ythickness : 0));
@@ -392,7 +392,7 @@ static void Dw_button_draw (DwWidget *widget,
 
    if (button->child &&
        p_Dw_widget_intersect (button->child, area, &child_area))
-      a_Dw_widget_draw (button->child, &child_area, event);
+      p_Dw_widget_draw (button->child, &child_area, event);
 }
 
 
