@@ -21,6 +21,7 @@
 
 typedef struct
 {
+   gchar *name;
    DwWidget *widget;
    gint32 y;
 } GtkDwViewportAnchor;
@@ -523,12 +524,24 @@ void a_Dw_gtk_viewport_add_dw (GtkDwViewport *viewport,
  **************************************************/
 
 /*
- * This function only *recognizes* that the top-level Dw widget is to
- * be removed. It is called by Dw_widget_shutdown.
- * Don't use this function directly!
+ * This function only *recognizes* that the top-level Dw widget has
+ * been destroyed, and only removes it from the viewport. It is called
+ * by Dw_widget_destroy. Don't use this function directly!
+ *
+ * N.b.: a widget, which stores anchors, should already have removed
+ * them from the widget, by calling a_Dw_gtk_viewport_remove_anchor().
+ * A good place for this is the implementation of GtkObject::destroy,
+ * the call to the parent destroy method should then be done at the
+ * end of the function, since Dw_widget_destroy calls this function.
  */
 void Dw_gtk_viewport_remove_dw (GtkDwViewport *viewport)
 {
+   /* Test, that all anchors have been removed properly. */
+   gint num_anchors_left = g_hash_table_size (viewport->anchors_table);
+   /* g_assert (num_anchors_left == 0); */
+   if (num_anchors_left != 0)
+      g_warning ("%d anchors left", num_anchors_left);
+
    a_Findtext_state_set_widget (viewport->findtext_state, NULL);
    viewport->child = NULL;
    Dw_gtk_viewport_remove_anchor (viewport);
@@ -779,9 +792,6 @@ DwWidget* Dw_gtk_viewport_widget_at_point (GtkDwViewport *viewport,
  *************/
 
 /*
- * Add or change an anchor.
- * The widget is responsible for storing a copy of name.
- *
  * todo: Currently, no horizontal scrolling is done. This is generally
  * possible, DW_HPOS_INTO_VIEW should be used for this, but it is
  * rather complicated to determine the width of an anchor. This would
@@ -789,31 +799,109 @@ DwWidget* Dw_gtk_viewport_widget_at_point (GtkDwViewport *viewport,
  * would have to have two kinds of content types (opening and closing
  * anchor), and some changes in the HTML parser are necessary.
  */
-void p_Dw_gtk_viewport_put_anchor (DwWidget *widget,
-                                   gchar *name,
-                                   gint32 y)
+
+/*
+ * Add an anchor, and assign a position for it. For all widgets
+ * directly or indirectly assigned to a viewports, anchors must be
+ * unique, this is tested. "name" is copied, so no strdup is
+ * neccessary for the caller.
+ *
+ * Return the copy on success, or NULL, when this anchor had already
+ * been added to the widget tree.
+ *
+ * The viewport gets the responsibility to free "name".
+ */
+gchar* p_Dw_gtk_viewport_add_anchor (DwWidget *widget,
+                                     const gchar *name,
+                                     gint32 y)
+{
+   GtkDwViewport *viewport;
+   GtkDwViewportAnchor *anchor;
+
+   _MSG("new anchor %p/'%s' -> %d\n", widget, name, y);
+
+   g_return_val_if_fail (widget->viewport != NULL, NULL);
+   viewport = GTK_DW_VIEWPORT (widget->viewport);
+
+   if (g_hash_table_lookup_extended (viewport->anchors_table, name, NULL,NULL))
+      /* Anchor does already exist. */
+      return NULL;
+   else {
+      anchor = g_new (GtkDwViewportAnchor, 1);
+      anchor->name = g_strdup (name);
+      anchor->widget = widget;
+      anchor->y = y;
+      
+      g_hash_table_insert (viewport->anchors_table, anchor->name, anchor);
+      Dw_gtk_viewport_update_anchor (viewport);
+      
+      return anchor->name;
+   }
+}
+
+
+/*
+ * Assign a position for an already existing anchor.
+ */
+void p_Dw_gtk_viewport_change_anchor (DwWidget *widget,
+                                      gchar *name,
+                                      gint32 y)
 {
    GtkDwViewport *viewport;
    GtkDwViewportAnchor *anchor;
    gpointer tmp_anchor;
+   gboolean exists;
 
+   _MSG("changing anchor %p/'%s' -> %d\n", widget, name, y);
+
+   g_return_if_fail (widget->viewport != NULL);
    viewport = GTK_DW_VIEWPORT (widget->viewport);
-   _MSG("  name(%p)=[%s]\n", name, name);
-   if (g_hash_table_lookup_extended (viewport->anchors_table, name, NULL,
-                                     &tmp_anchor)) {
-      anchor = tmp_anchor;
-   } else {
-      anchor = g_new (GtkDwViewportAnchor, 1);
-      g_hash_table_insert (viewport->anchors_table, name, anchor);
-   }
 
-   anchor->widget = widget;
+   exists =
+      g_hash_table_lookup_extended (viewport->anchors_table, name, NULL,
+                                    &tmp_anchor);
+   g_return_if_fail(exists);
+
+   anchor = tmp_anchor;
+   g_return_if_fail(anchor->widget == widget);
+
    anchor->y = y;
 
    Dw_gtk_viewport_update_anchor (viewport);
 }
 
 
+/*
+ * Remove an anchor from the table in the viewport. Notice that "name"
+ * is freed here.
+ */
+void p_Dw_gtk_viewport_remove_anchor (DwWidget *widget,
+                                      gchar *name)
+{
+   GtkDwViewport *viewport;
+   GtkDwViewportAnchor *anchor;
+   gpointer tmp_anchor;
+   gboolean exists;
+
+   _MSG("removing anchor %p/'%s'\n", widget, name);
+
+   g_return_if_fail (widget->viewport != NULL);
+   viewport = GTK_DW_VIEWPORT (widget->viewport);
+
+   exists = 
+      g_hash_table_lookup_extended (viewport->anchors_table, name, NULL,
+                                    &tmp_anchor);
+   g_return_if_fail(exists);
+
+   anchor = tmp_anchor;
+   g_return_if_fail(anchor->widget == widget);
+   
+   g_hash_table_remove (viewport->anchors_table, name);
+   g_free (anchor->name);
+   g_free (anchor);
+}
+
+
 /*
  * Used by Dw_gtk_viewport_update_anchor_idle.
  */
