@@ -620,7 +620,7 @@ static void Dw_image_scale (DwImage *image)
 
    if (image->width != w || image->height != h) {
       /* scaled image */
-      image->scaled_buffer = g_malloc (3 * w * h);
+      image->scaled_buffer = g_malloc ((gulong)3 * w * h);
 
       for (y = 0; y < image->height; y++)
          Dw_image_scale_row (image, y);
@@ -649,7 +649,7 @@ static void Dw_image_scale_row (DwImage *image, gint y_src)
       dest1 = image->scaled_buffer + 3 * y_dest1 * w_dest;
 
       if (w_src == w_dest)
-         memcpy (dest1, src, 3 * w_src);
+         memcpy (dest1, src, 3 * (size_t)w_src);
       else if (w_dest > w_src) {
          delta = w_src / 2;
          x_src = 0;
@@ -683,7 +683,7 @@ static void Dw_image_scale_row (DwImage *image, gint y_src)
       /* The other lines are simply copied. */
       for (y_dest = y_dest1 + 1; y_dest < y_dest2; y_dest++) {
          dest = image->scaled_buffer + 3 * y_dest * w_dest;
-         memcpy (dest, dest1, 3 * w_dest);
+         memcpy (dest, dest1, 3 * (size_t)w_dest);
       }
    }
 }
