@@ -10,12 +10,14 @@
  */
 
 #include "dw_list_item.h"
+#include "debug.h"
 
 static void   Dw_list_item_class_init    (DwListItemClass *klass);
 static void   Dw_list_item_init          (DwListItem *list_item);
 
 static gint32 Dw_list_item_get_value     (DwAlignedPage *aligned_page);
 static void   Dw_list_item_set_max_value (DwAlignedPage *aligned_page,
+                                          gint32 max_value,
                                           gint32 value);
 
 /*
@@ -54,6 +56,7 @@ DwWidget* a_Dw_list_item_new (DwListItem *ref_list_item)
    GtkObject *object;
 
    object = gtk_object_new (DW_TYPE_LIST_ITEM, NULL);
+   DBG_OBJ_CREATE (object, "DwListItem");
    p_Dw_aligned_page_set_ref_page (DW_ALIGNED_PAGE (object),
                                    (DwAlignedPage*)ref_list_item);
    return DW_WIDGET (object);
@@ -99,12 +102,13 @@ static gint32 Dw_list_item_get_value (DwAlignedPage *aligned_page)
  * Implementation of DwAlignedPage::set_max_value.
  */
 static void Dw_list_item_set_max_value (DwAlignedPage *aligned_page,
+                                        gint32 max_value,
                                         gint32 value)
 {
    DwPage *page = DW_PAGE (aligned_page);
 
-   page->inner_padding = value;
-   page->line1_offset = - Dw_list_item_get_value (aligned_page);
+   page->inner_padding = max_value;
+   page->line1_offset = - value;
    p_Dw_widget_queue_resize (DW_WIDGET (aligned_page), 0, TRUE);
 }
 
