@@ -327,17 +327,17 @@ static void Dw_page_class_init (DwPageClass *klass)
    klass->word_wrap = Dw_page_real_word_wrap;
 }
 
-
 /*
  * Destroy the page (standard Gtk+ function)
  */
 static void Dw_page_destroy (GtkObject *object)
 {
+   DwWidget *widget = DW_WIDGET (object);
    DwPage *page = DW_PAGE (object);
    DwPageWord *word;
    gint i;
 
-   //DEBUG_MSG (10, "Dw_page_destroy\n");
+   _MSG ("Dw_page_destroy\n");
 
    for (i = 0; i < page->num_words; i++) {
       word = &page->words[i];
@@ -346,7 +346,9 @@ static void Dw_page_destroy (GtkObject *object)
       else if (word->content.type == DW_CONTENT_TEXT)
          g_free (word->content.data.text);
       else if (word->content.type == DW_CONTENT_ANCHOR)
-         g_free (word->content.data.anchor);
+         /* This also frees the names (see  p_Dw_gtk_viewport_remove_anchor()
+          * and related). */
+         p_Dw_gtk_viewport_remove_anchor(widget, word->content.data.anchor);
 
       a_Dw_style_unref (word->style);
       a_Dw_style_unref (word->space_style);
@@ -626,7 +628,7 @@ static void Dw_page_size_allocate (DwWidget *widget,
             break;
 
          case DW_CONTENT_ANCHOR:
-            p_Dw_gtk_viewport_put_anchor
+            p_Dw_gtk_viewport_change_anchor
                (widget, word->content.data.anchor,
                 Dw_page_line_total_y_offset_allocation (page, line,
                                                         allocation));
@@ -1280,7 +1282,7 @@ static void Dw_page_rewrap (DwPage *page)
                 page->num_lines - 1, word_index, page->num_words);
 
       if ( word->content.type == DW_CONTENT_ANCHOR )
-         p_Dw_gtk_viewport_put_anchor
+         p_Dw_gtk_viewport_change_anchor
             (widget, word->content.data.anchor,
              Dw_page_line_total_y_offset (page,
                                           &page->lines[page->num_lines - 1]));
@@ -2040,24 +2042,40 @@ void a_Dw_page_add_widget (DwPage *page,
 
 
 /*
- * Add an anchor to the page. name is copied, so no strdup is neccessary for
+ * Add an anchor to the page. "name" is copied, so no strdup is neccessary for
  * the caller.
+ *
+ * Return TRUE on success, and FALSE, when this anchor had already been
+ * added to the widget tree.
  */
-void a_Dw_page_add_anchor (DwPage *page, const char *name, DwStyle *style)
+gboolean a_Dw_page_add_anchor (DwPage *page, const gchar *name, DwStyle *style)
 {
    DwPageWord *word;
+   char *copy;
+   gint32 y;
 
-   word = Dw_page_add_word (page, 0, 0, 0, style);
-   word->content.type = DW_CONTENT_ANCHOR;
-   word->content.data.anchor = g_strdup (name);
-   Dw_page_word_wrap (page, page->num_words - 1);
+   if (page->num_lines == 0)
+      y = 0;
+   else
+      y = Dw_page_line_total_y_offset_i (page, page->num_lines - 1);
 
-   DBG_OBJ_ARRSET_STR (page, "words.%d.content.anchor", page->num_words - 1,
-                       word->content.data.anchor);
+   /*
+    * Since an anchor does not take any space, it is safe to call 
+    * p_Dw_gtk_viewport_add_anchor already here.
+    */
+   if ((copy = p_Dw_gtk_viewport_add_anchor(DW_WIDGET(page), name, y)) == NULL)
+      return FALSE;
+   else {
+      word = Dw_page_add_word (page, 0, 0, 0, style);
+      word->content.type = DW_CONTENT_ANCHOR;
+      word->content.data.anchor = copy;
+      Dw_page_word_wrap (page, page->num_words - 1);
+      
+      DBG_OBJ_ARRSET_STR (page, "words.%d.content.anchor", page->num_words - 1,
+                          word->content.data.anchor);
 
-   p_Dw_gtk_viewport_put_anchor
-      (DW_WIDGET(page), word->content.data.anchor,
-       Dw_page_line_total_y_offset_i (page, page->num_lines - 1));
+      return TRUE;
+   }
 }
 
 
