@@ -876,6 +876,14 @@ void a_Dw_table_add_cell (DwTable *table,
    gint col, row, old_num_cols, old_num_rows;
    DwTableChild *child;
 
+   /* We limit the values for colspan and rowspan to 50, to avoid
+    * attacks by malicious web pages.
+    * todo: Some warnings would be nice. */
+   if (colspan > 50)
+      colspan = 50;
+   if (rowspan > 50)
+      rowspan = 50;
+
    if (table->num_rows == 0)
       /* to prevent a crash */
       a_Dw_table_add_row (table, NULL);
