@@ -106,8 +106,7 @@ static gchar *Findtext_compare (gchar *haystack, gchar *needle,
 /*
  * Creates a new key from a string.
  */
-FindtextKey *Findtext_key_new (gchar *string,
-                               gboolean case_sens)
+static FindtextKey *Findtext_key_new (gchar *string, gboolean case_sens)
 {
    FindtextKey *key;
    gint i, j, *splitpos;
@@ -192,7 +191,7 @@ FindtextKey *Findtext_key_new (gchar *string,
 /*
  * Frees the memory allocated by a key.
  */
-void Findtext_key_destroy (FindtextKey *key)
+static void Findtext_key_destroy (FindtextKey *key)
 {
    int i;
    for (i = 0; i < key->len; i++)
