@@ -16,7 +16,7 @@ int a_History_set_title(gint idx, const gchar *title);
 DilloUrl *a_History_get_url(gint idx);
 const gchar *a_History_get_title(gint idx);
 const gchar *a_History_get_title_by_url(DilloUrl *url, gint force);
-void a_History_free();
+void a_History_free(void);
 
 
 #ifdef __cplusplus
