@@ -97,7 +97,7 @@ typedef enum {
 
 
 struct _DilloHtmlState {
-   char *tag;
+   char *tag_name;
    DwStyle *style, *table_cell_style;
    DilloHtmlParseMode parse_mode;
    DilloHtmlTableMode table_mode;
@@ -196,7 +196,7 @@ struct _DilloHtml {
    DilloHtmlLB *linkblock;
    gchar *Start_Buf;
    size_t Start_Ofs;
-   size_t CurrTagOfs, CurrTagIdx;
+   size_t CurrTagOfs;
    size_t OldTagOfs, OldTagLine;
 
    DilloHtmlState *stack;
@@ -216,6 +216,13 @@ struct _DilloHtml {
    gboolean PrevWasOpenTag; /* Flag to help deferred parsing of white space */
    gboolean SPCPending;     /* Flag to help deferred parsing of white space */
    gboolean InVisitedLink;  /* used to 'contrast_visited_colors' */
+   gboolean ReqTagClose;    /* Flag to help handling bad-formed HTML */
+   gboolean CloseOneTag;    /* Flag to help Html_tag_cleanup_at_close() */
+   gboolean TagSoup;        /* Flag to enable the parser's cleanup functions */
+   gchar *NameVal;          /* used for validation of "NAME" and "ID" in <A> */
+
+   /* element counters: used for validation purposes */
+   guchar Num_HTML, Num_HEAD, Num_BODY, Num_TITLE;
 
    GString *attr_data;
 
