@@ -711,7 +711,7 @@ static GtkWidget *Interface_progressbox_new(BrowserWindow *bw, gint vertical)
  * Hide/Unhide this bw's control panels.
  * toggle: Flag [toggle or set].
  */
-void Interface_toggle_panel(BrowserWindow *bw, gint toggle)
+static void Interface_toggle_panel(BrowserWindow *bw, gint toggle)
 {
    if (toggle)
       bw->fullwindow = !bw->fullwindow;
@@ -912,7 +912,7 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
       gtk_box_pack_start(GTK_BOX(box1), handlebox, FALSE, FALSE, 0);
       gtk_widget_show(handlebox);
 
-   } else if (prefs.panel_size == 2) {
+   } else if (prefs.panel_size == 2 || prefs.panel_size == 3) {
       handlebox = gtk_handle_box_new();
       bw->PanelHandles = g_slist_append(bw->PanelHandles, handlebox);
       hbox = gtk_hbox_new(FALSE, 0);
@@ -931,8 +931,8 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
       bw->PanelHandles = g_slist_append(bw->PanelHandles, handlebox);
       gtk_container_border_width(GTK_CONTAINER(handlebox), 4);
       hbox = gtk_hbox_new(FALSE, 0);
-      toolbar = Interface_toolbar_new(bw, 1);
-      progbox = Interface_progressbox_new(bw, 1);
+      toolbar = Interface_toolbar_new(bw, (prefs.panel_size == 3));
+      progbox = Interface_progressbox_new(bw, (prefs.panel_size == 3));
       gtk_box_pack_start(GTK_BOX(hbox), toolbar, TRUE, TRUE, 0);
       gtk_widget_show(toolbar);
       gtk_box_pack_start(GTK_BOX(hbox), progbox, FALSE, FALSE, 0);
@@ -1184,7 +1184,7 @@ void a_Interface_msg(BrowserWindow *bw, const char *format, ... )
 
    if ( bw ) {
       va_start(argp, format);
-      vsnprintf(msg, 1024, format, argp);
+      g_vsnprintf(msg, 1024, format, argp);
       va_end(argp);
       gtk_label_set_text(GTK_LABEL(bw->status), msg);
       bw->status_is_link = 0;
@@ -1573,7 +1573,8 @@ static void Interface_make_question_dialog(
    for (i = 1; i < 6 && answer[i].alt_str; ++i) {
       button = gtk_button_new_with_label(answer[i].alt_str);
       gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-         Interface_question_dialog_set_answer_cb, (gpointer)&answer[i]);
+         (GtkSignalFunc) Interface_question_dialog_set_answer_cb,
+         (gpointer)&answer[i]);
       gtk_signal_connect(GTK_OBJECT(button), "clicked",
          (GtkSignalFunc) Interface_destroy_window, DialogWindow);
       gtk_widget_show(button);
@@ -2000,7 +2001,7 @@ void a_Interface_message_window(const char *title, const char *format, ... )
    va_list argp;
 
    va_start(argp, format);
-   vsnprintf(msg, 1024, format, argp);
+   g_vsnprintf(msg, 1024, format, argp);
    va_end(argp);
 
    window = gtk_window_new(GTK_WINDOW_DIALOG);
