@@ -24,7 +24,12 @@
 #include <gtk/gtk.h>
 #include <setjmp.h>
 
+/* avoid a redefinition of HAVE_STDLIB_H with old jpeglib.h */
+#ifdef HAVE_STDLIB_H
+#  undef HAVE_STDLIB_H
+#endif
 #include <jpeglib.h>
+
 #include "image.h"
 #include "web.h"
 #include "cache.h"
@@ -80,6 +85,9 @@ static void Jpeg_write(DilloJpeg *jpeg, void *Buf, guint BufSize);
 static void Jpeg_close(DilloJpeg *jpeg, CacheClient_t *Client);
 METHODDEF(void) Jpeg_errorexit (j_common_ptr cinfo);
 
+/* exported function */
+DwWidget *a_Jpeg_image(const char *Type, void *P, CA_Callback_t *Call,
+                       void **Data);
 
 
 /* this is the routine called by libjpeg when it detects an error. */
@@ -148,8 +156,8 @@ static boolean fill_input_buffer(j_decompress_ptr cinfo)
    DEBUG_MSG(5, "fill_input_buffer\n");
 #if 0
    if (!cinfo->src->bytes_in_buffer) {
-      DEBUG_MSG(5, "fill_input_buffer: %d bytes in buffer\n",
-                cinfo->src->bytes_in_buffer);
+      DEBUG_MSG(5, "fill_input_buffer: %ld bytes in buffer\n",
+                (glong)cinfo->src->bytes_in_buffer);
 
       jpeg->Start_Ofs = (gulong) jpeg->cinfo.src->next_input_byte -
          (gulong) jpeg->Data;
@@ -176,9 +184,10 @@ static void skip_input_data(j_decompress_ptr cinfo, glong num_bytes)
       return;
    jpeg = ((my_source_mgr *) cinfo->src)->jpeg;
 
-   DEBUG_MSG(5, "skip_input_data: Start_Ofs = %d, num_bytes = %d,"
-             " %d bytes in buffer\n",
-             jpeg->Start_Ofs, (gint) num_bytes, cinfo->src->bytes_in_buffer);
+   DEBUG_MSG(5, "skip_input_data: Start_Ofs = %lu, num_bytes = %ld,"
+             " %ld bytes in buffer\n",
+             (gulong)jpeg->Start_Ofs, num_bytes,
+             (glong)cinfo->src->bytes_in_buffer);
 
    cinfo->src->next_input_byte += num_bytes;
    if (num_bytes < (glong)cinfo->src->bytes_in_buffer) {
@@ -247,8 +256,8 @@ static void Jpeg_write(DilloJpeg *jpeg, void *Buf, guint BufSize)
    JSAMPLE *array[1];
    gint num_read;
 
-   DEBUG_MSG(5, "Jpeg_write: (%p) Bytes in buff: %ld Ofs: %d\n", jpeg,
-             (glong) BufSize, jpeg->Start_Ofs);
+   DEBUG_MSG(5, "Jpeg_write: (%p) Bytes in buff: %ld Ofs: %lu\n", jpeg,
+             (glong) BufSize, (gulong)jpeg->Start_Ofs);
 
    /* See if we are supposed to skip ahead. */
    if (BufSize <= jpeg->Start_Ofs)
@@ -279,8 +288,8 @@ static void Jpeg_write(DilloJpeg *jpeg, void *Buf, guint BufSize)
             DEBUG_MSG(5, "jpeg: can't handle %d component images\n",
                       jpeg->cinfo.num_components);
          a_Dicache_set_parms(jpeg->url, jpeg->version, jpeg->Image,
-                             jpeg->cinfo.image_width,
-                             jpeg->cinfo.image_height,
+                             (guint)jpeg->cinfo.image_width,
+                             (guint)jpeg->cinfo.image_height,
                              type);
 
          /* decompression step 4 (see libjpeg.doc) */
