@@ -14,14 +14,14 @@
  * (First, allocate an 'alloc_step' sized chunk, after that, double the
  *  list size --to make it faster)
  */
-#define a_List_resize(list,num_items,item_size,alloc_step) \
+#define a_List_resize(list,num_items,alloc_step) \
    if ( !list ) { \
-      list = g_malloc(alloc_step * item_size); \
+      list = g_malloc(alloc_step * sizeof((*list))); \
    } \
    if ( num_items >= alloc_step ){ \
       while ( num_items >= alloc_step ) \
          alloc_step <<= 1; \
-      list = g_realloc(list, alloc_step * item_size); \
+      list = g_realloc(list, alloc_step * sizeof((*list))); \
    }
 
 
@@ -30,8 +30,8 @@
  *
  * Make sure there's space for one more item within the list.
  */
-#define a_List_add(list,num_items,item_size,alloc_step) \
-   a_List_resize(list,num_items,item_size,alloc_step)
+#define a_List_add(list,num_items,alloc_step) \
+   a_List_resize(list,num_items,alloc_step)
 
 
 /*
