@@ -224,14 +224,18 @@ gint *a_Misc_strsplitposdup(gint *pos)
  * only checks the few mime types that Dillo supports.
  *
  * 'Data' is a pointer to the first bytes of the raw data.
+ *
+ * Return value: (0 on success, 1 on doubt, 2 on lack of data).
  */
-const gchar *a_Misc_get_content_type_from_data(void *Data, size_t Size)
+gint a_Misc_get_content_type_from_data(void *Data, size_t Size,
+                                       const gchar **PT)
 {
    static const gchar *Types[] = {
       "application/octet-stream",
       "text/html", "text/plain",
       "image/gif", "image/png", "image/jpeg",
    };
+   gint st = 1;    /* default to "doubt' */
    gint Type = 0;
    gchar *p = Data;
    size_t i;
@@ -246,16 +250,19 @@ const gchar *a_Misc_get_content_type_from_data(void *Data, size_t Size)
        (Size - i >= 17 && !g_strncasecmp(p+i, "<!-- HTML listing", 17))) {
 
       Type = 1;
-
+      st = 0;
    /* Images */
    } else if (Size >= 4 && !g_strncasecmp(p, "GIF8", 4)) {
       Type = 3;
+      st = 0;
    } else if (Size >= 4 && !g_strncasecmp(p, "\x89PNG", 4)) {
       Type = 4;
+      st = 0;
    } else if (Size >= 2 && !g_strncasecmp(p, "\xff\xd8", 2)) {
       /* JPEG has the first 2 bytes set to 0xffd8 in BigEndian - looking
        * at the character representation should be machine independent. */
       Type = 5;
+      st = 0;
 
    /* Text */
    } else {
@@ -266,9 +273,12 @@ const gchar *a_Misc_get_content_type_from_data(void *Data, size_t Size)
          if ((guchar) p[i] > 127)
             break;
       Type = (i < 12 || i < Size) ? 0 : 2;
+      if (i >= 12 && i == Size)
+         st = 0;
    }
 
-   return (Types[Type]);
+   *PT = Types[Type];
+   return st;
 }
 
 /*
