@@ -11,7 +11,8 @@ gchar *a_Misc_strsep(char **orig, const char *delim);
 #define d_strsep a_Misc_strsep
 gint *a_Misc_strsplitpos(const gchar *str, const gchar *delim);
 gint *a_Misc_strsplitposdup(gint *pos);
-const gchar *a_Misc_get_content_type_from_data(void *Data, size_t Size);
+gint a_Misc_get_content_type_from_data(void *Data, size_t Size,
+                                       const gchar **PT);
 gint a_Misc_parse_geometry(gchar *geom, gint *x, gint *y, gint *w, gint *h);
 gchar *a_Misc_encode_base64(const gchar *in);
 
