@@ -107,6 +107,14 @@ struct _DilloPng {
 #define BLACK     0
 #define WHITE     255
 
+/*
+ * Forward declarations
+ */
+/* exported function */
+DwWidget *a_Png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
+                      void **Data);
+
+
 static
 void Png_error_handling(png_structp png_ptr, png_const_charp msg)
 {
@@ -203,7 +211,8 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
 
    /* Initialize the dicache-entry here */
    a_Dicache_set_parms(png->url, png->version, png->Image,
-                       png->width, png->height, DILLO_IMG_TYPE_RGB);
+                       (guint)png->width, (guint)png->height,
+                       DILLO_IMG_TYPE_RGB);
 }
 
 static void
@@ -225,7 +234,8 @@ static void
    switch (png->channels) {
    case 3:
       a_Dicache_write(png->Image, png->url, png->version,
-                      png->image_data + (row_num * png->rowbytes), 0, row_num);
+                      png->image_data + (row_num * png->rowbytes),
+                      0, (guint)row_num);
       break;
    case 4:
      {
@@ -262,7 +272,7 @@ static void
            }
         }
         a_Dicache_write(png->Image, png->url, png->version,
-                        png->linebuf, 0, row_num);
+                        png->linebuf, 0, (guint)row_num);
         break;
      }
    default:
@@ -379,7 +389,7 @@ static void Png_callback(int Op, CacheClient_t *Client)
             png_process_data( png->png_ptr,
                               png->info_ptr,
                               png->ipbuf + png->ipbufstart,
-                              DATASIZE);
+                              (png_size_t)DATASIZE);
 
             png->ipbufstart += DATASIZE;
          }
