@@ -161,10 +161,12 @@ static guint Prefs_parser(GScanner *scanner)
    case DRC_TOKEN_PANEL_SIZE:
       if (!g_strcasecmp(scanner->value.v_string, "tiny"))
          prefs.panel_size = 1;
-      else if (!g_strcasecmp(scanner->value.v_string, "medium"))
+      else if (!g_strcasecmp(scanner->value.v_string, "small"))
          prefs.panel_size = 2;
-      else /* default to "large" */
+      else if (!g_strcasecmp(scanner->value.v_string, "medium"))
          prefs.panel_size = 3;
+      else /* default to "large" */
+         prefs.panel_size = 4;
       break;
    case DRC_TOKEN_SMALL_ICONS:
       prefs.small_icons = (strcmp(scanner->value.v_string, "YES") == 0);
