@@ -88,10 +88,10 @@ struct _DilloPrefs {
    gchar **no_proxy_vec;
    DilloUrl *start_page;
    DilloUrl *home;
-   guint32 link_color;
-   guint32 visited_color;
-   guint32 bg_color;
-   guint32 text_color;
+   gint32 link_color;
+   gint32 visited_color;
+   gint32 bg_color;
+   gint32 text_color;
    gboolean allow_white_bg;
    gboolean use_oblique;
    gboolean force_my_colors;
