@@ -87,7 +87,7 @@ const gchar *a_Url_hostname(const DilloUrl *u)
    if (!url->hostname && url->authority) {
       if ((p = strchr(url->authority, ':'))) {
          url->port = strtol(p + 1, NULL, 10);
-         url->hostname = g_strndup(url->authority, p - url->authority);
+         url->hostname = g_strndup(url->authority,(guint)(p - url->authority));
       } else
          url->hostname = url->authority;
    }
@@ -124,7 +124,7 @@ static DilloUrl *Url_object_new(const gchar *uri_str)
       s = p + 2;
       p = strpbrk(s, "/?#");
       if (p) {
-         memmove(s - 2, s, MAX(p - s, 1));
+         memmove(s - 2, s, (size_t)MAX(p - s, 1));
          url->authority = s - 2;
          p[-2] = 0;
          s = p;
