@@ -35,8 +35,8 @@ struct _DilloWeb {
 DilloWeb* a_Web_new (const DilloUrl* url);
 gint a_Web_valid(DilloWeb *web);
 void a_Web_free (DilloWeb*);
-DwWidget* a_Web_dispatch_by_type (const char *Type, DilloWeb *web,
-                                  CA_Callback_t *Call, void **Data);
+gint a_Web_dispatch_by_type (const char *Type, DilloWeb *web,
+                             CA_Callback_t *Call, void **Data);
 
 #ifdef __cplusplus
 }
